from pyqchem import get_output_from_qchem, QchemInput
from pyqchem.parsers.parser_frequencies import basic_frequencies
from pyqchem.parsers.parser_optimization import basic_optimization
from pyqchem.parsers.parser_irc import basic_irc
from pyqchem.structure import Structure
from pyqchem.file_io import write_structure_to_xyz
import matplotlib.pyplot as plt


# define molecule
coordinates = [[ 0.6268743917,   -0.1366254266,   -0.0000000000],
               [ 0.3711605704,    1.0377672206,    0.0000000000],
               [-0.5903438458,   -0.0311449516,    0.0000000000]]

symbols = ['C', 'H', 'N']

molecule = Structure(coordinates=coordinates,
                     symbols=symbols,
                     charge=0,
                     multiplicity=1)

# Transition state optimization
qc_input = QchemInput(molecule,
                      jobtype='ts',
                      exchange='hf',
                      basis='sto-3g',
                      geom_opt_tol_gradient=300,
                      geom_opt_tol_energy=100,
                      geom_opt_tol_displacement=1200,
                      geom_opt_max_cycles=50,  # reduce this number to test not convergence case
                      )

opt_data, ee = get_output_from_qchem(qc_input,
                                     processors=4,
                                     parser=basic_optimization,
                                     force_recalculation=False,
                                     read_fchk=True,
                                     store_full_output=True)


print('Transition state')
print(opt_data['optimized_molecule'])

# frequencies calculation
qc_input = QchemInput(opt_data['optimized_molecule'],
                      jobtype='freq',
                      exchange='hf',
                      basis='sto-3g',
                      sym_ignore=True,
                      symmetry=False,
                      scf_guess=ee['coefficients'])

freq_data = get_output_from_qchem(qc_input,
                                  processors=4,
                                  # force_recalculation=True,
                                  parser=basic_frequencies,
                                  store_full_output=True)

# IRC calculation
qc_input = QchemInput(opt_data['optimized_molecule'],
                      jobtype='rpath',
                      exchange='hf',
                      basis='sto-3g',
                      rpath_max_cycles=30,
                      scf_guess=ee['coefficients'],
                      hessian=freq_data['hessian']
                      )

irc_data = get_output_from_qchem(qc_input,
                                 processors=4,
                                 parser=basic_irc,
                                 store_full_output=True
                                 )

# write coordinates into file
write_structure_to_xyz([step['molecule'] for step in irc_data['irc_forward']][::-1] +
                       [step['molecule'] for step in irc_data['irc_backward']], 'irc.xyz')

# plot SCF energies
energies_f = [step['energy'] for step in irc_data['irc_forward']]
energies_b = [step['energy'] for step in irc_data['irc_backward']]

plt.plot(energies_f[::-1] + energies_b)
plt.xticks([], [])
plt.xlabel('Intrinsic reaction coordinate')
plt.ylabel('Energy (Hartree)')
plt.show()



from pyqchem import get_output_from_qchem, QchemInput
from pyqchem.structure import Structure
from pyqchem.parsers.basic import basic_parser_qchem


remote_data_pc = {'hostname': '111.222.333.44',
                  'port': 22,
                  'username': 'abel',
                  'password': '**************',
                  'allow_agent': False,
                  'look_for_keys': False,
                  'precommand': ['module load qchem/qchem_group'],
                  'remote_scratch': '/home/abel/'}


ethene = [[0.0,  0.0000,   0.65750],
          [0.0,  0.0000,  -0.65750],
          [0.0,  0.92281,  1.22792],
          [0.0, -0.92281,  1.22792],
          [0.0, -0.92281, -1.22792],
          [0.0,  0.92281, -1.22792]]

symbols = ['C', 'C', 'H', 'H', 'H', 'H']

# create molecule
molecule = Structure(coordinates=ethene,
                     symbols=symbols,
                     charge=0,
                     multiplicity=1)

# create Q-Chem input
qc_input = QchemInput(molecule,
                      jobtype='sp',
                      exchange='hf',
                      basis='sto-3g')

print(qc_input.get_txt())


# get data from Q-Chem calculation
output = get_output_from_qchem(qc_input,
                               processors=1,
                               force_recalculation=True,
                               remote=remote_data_pc,
                               scratch='/Users/abel/Scratch/export/',
                               parser=basic_parser_qchem
                               )

# Get data
print('OUTPUT')
print(output)

print('the energy is: ', output['scf_energy'])


from pyqchem import Structure, QchemInput, get_output_from_qchem
from pyqchem.qchem_core import redefine_calculation_data_filename
from pyqchem.parsers.parser_optimization import basic_optimization
from pyqchem.parsers.parser_cis import basic_cis
from pyqchem.parsers.parser_rasci import parser_rasci
from pyqchem.basis import get_basis_from_ccRepo
from pyqchem.file_io import build_fchk
from pyqchem.errors import OutputError

redefine_calculation_data_filename('soc_benzo_2_ras.pkl')

coordinates = [[-0.49800797,  0.41832669,  0.00000000],
               [ 0.72930903,  0.41832669,  0.00000000],
               [-1.31920411,  1.72110714,  0.00000000],
               [-2.71298836,  1.66723263,  0.00000000],
               [-0.66872148,  2.95532032,  0.00026620],
               [-3.45673426,  2.84793521, -0.00001025],
               [-1.41217483,  4.13550038, -0.00026053],
               [ 0.43007141,  2.99754789,  0.00046401],
               [-2.80629666,  4.08169051, -0.00073971],
               [-4.55559312,  2.80544226,  0.00024612],
               [-0.89955468,  5.10831249, -0.00038580],
               [-3.39213342,  5.01230158, -0.00037769],
               [-1.31920411, -0.88445377,  0.00005409],
               [-2.71298836, -0.83057926,  0.00005185],
               [-0.66872148, -2.11866695, -0.00016087],
               [-3.45673426, -2.01128184,  0.00011112],
               [-1.41217483, -3.29884699,  0.00041485],
               [ 0.43007141, -2.16089453, -0.00035693],
               [-2.80629666, -3.24503710,  0.00089180],
               [-4.55559312, -1.96878890, -0.00014701],
               [-0.89955468, -4.27165909,  0.00058051],
               [-3.39213342, -4.17564818,  0.00056841],
               [-3.43149206,  0.53153375, -0.00072728],
               [-4.06231571,  0.55638246, -0.86463934],
               [-4.06411994,  0.55641830,  0.86186343]]

symbols = ['C', 'O', 'C', 'C', 'C', 'C', 'C', 'H', 'C', 'H', 'H', 'H',
           'C', 'C', 'C', 'C', 'C', 'H', 'C', 'H', 'H', 'H', 'C', 'H', 'H']

molecule = Structure(coordinates=coordinates,
                     symbols=symbols,
                     charge=0,
                     multiplicity=1)

basis_custom_repo = get_basis_from_ccRepo(molecule, 'cc-pVDZ')

qc_input = QchemInput(molecule,
                      jobtype='opt',
                      exchange='b3lyp',
                      basis='sto-3g',
                      geom_opt_tol_gradient=300,
                      geom_opt_tol_energy=100,
                      geom_opt_tol_displacement=1200,
                      geom_opt_max_cycles=50,
                      # n_frozen_core=0,
                      )

try:
    parsed_data, electronic_structure = get_output_from_qchem(qc_input,
                                                              processors=4,
                                                              parser=basic_optimization,
                                                              read_fchk=True,
                                                              store_full_output=True
                                                              )
except OutputError as e:
    print(e.error_lines)
    exit()

opt_molecule = parsed_data['optimized_molecule']

print('Optimized molecule')
print(opt_molecule.get_xyz())

basis_custom_repo = get_basis_from_ccRepo(molecule, 'cc-pVTZ')

qc_input = QchemInput(opt_molecule,
                      jobtype='sp',
                      exchange='b3lyp',
                      basis='sto-3g',
                      cis_n_roots=5,
                      cis_singlets=True,
                      cis_triplets=True,
                      calc_soc=1,
                      n_frozen_core=0,
                      )

print(qc_input.get_txt())

try:
    output, electronic_structure = get_output_from_qchem(qc_input,
                                                         processors=4,
                                                         # parser=basic_cis,
                                                         read_fchk=True,
                                                         store_full_output=True,
                                                         )
except OutputError as e:
    print(e.error_lines)
    exit()

print(output)
open('benzo_2.fchk', 'w').write(build_fchk(electronic_structure))



#!/usr/bin/env python3

from pyqchem.qchem_core import get_output_from_qchem, create_qchem_input
from pyqchem.parsers.parser_frequencies import basic_frequencies
from pyqchem.parsers.parser_optimization import basic_optimization
from pyqchem.structure import Structure

import numpy as np

import argparse


def get_rmsd(coor, coor2):
    coor = np.array(coor)
    coor2 = np.array(coor2)
    rms = np.average(np.square(coor - coor2))
    rmsd = np.sqrt(rms)
    return rmsd


# Argument parser
parser = argparse.ArgumentParser(description='normal modes')
parser.add_argument('filename', metavar='filename', type=str,
                    help='filename for output')
parser.add_argument('--show_plot', action='store_true',
                    help='perform test and show plots')
parser.add_argument('-t', metavar='temperature', type=float, default=300,
                    help='temperature to calculate normal mode amplitude', )


args = parser.parse_args()


coordinates = [[ 0.         ,  2.5593592,   -1.2291437],
               [ 0.         ,  1.2177072,   -0.7388171],
               [ 0.         ,  1.2539645,    0.6577044],
               [ 0.         ,  2.5457806,    1.0474186],
               [ 0.         ,  3.3773954,   -0.1013709],
               [ 0.         ,  0.       ,    1.5890149],
               [ 0.         , -1.2539645,    0.6577044],
               [ 0.         , -1.2177072,   -0.7388171],
               [ 0.         , -2.5593592,   -1.2291437],
               [ 0.         , -3.3773954,   -0.1013709],
               [ 0.         , -2.5457806,    1.0474186],
               [ 0.         , -2.9786512,   -2.6674481],
               [ 0.         ,  0.       ,   -1.4142026],
               [ 0.         , -2.9494503,    2.4837241],
               [ 0.         ,  2.9494503,    2.4837241],
               [ 0.         ,  2.9786512,   -2.6674481],
               [ 1.1494068  ,  0.       ,    2.3950298],
               [ -1.1494068 ,  0.       ,    2.3950298],
               [ 0.         ,  0.       ,   -2.5152032],
               [ 0.         ,  4.4782605,   -0.0840819],
               [ 0.         , -4.4782605,   -0.0840819],
               [ 0.         , -4.0761113,   -2.7556685],
               [ -0.8867106 , -2.6006386,   -3.199612],
               [ 0.8867106  , -2.6006386,   -3.199612],
               [ 0.8941856  , -2.5562337,    2.9920521],
               [ -0.8941856 , -2.5562337,    2.9920521],
               [ 0.         , -4.0461503,    2.5809354],
               [ -0.8867106 ,  2.6006386,   -3.199612],
               [ 0.         ,  4.0761113,   -2.7556685],
               [ 0.8867106  ,  2.6006386,   -3.199612],
               [ 0.         ,  4.0461503,    2.5809354],
               [ -0.8941856 ,  2.5562337,    2.9920521],
               [ 0.8941856  ,  2.5562337,    2.9920521]]

symbols = ['C', 'C', 'N', 'C', 'C', 'B', 'N', 'C', 'C', 'C', 'C', 'C', 'C', 'C', 'C',
           'C', 'F', 'F', 'H', 'H', 'H', 'H', 'H', 'H', 'H', 'H', 'H', 'H', 'H', 'H',
           'H', 'H', 'H']


molecule = Structure(coordinates=coordinates,
                     symbols=symbols,
                     charge=0,
                     multiplicity=1)

molecule.get_coordinates()

print('optimize molecule')

txt_input = create_qchem_input(molecule,
                               jobtype='opt',
                               method='b3lyp',
                               basis='cc-pVDZ',
                               geom_opt_tol_gradient=50,
                               geom_opt_coords=-1,
                               geom_opt_tol_displacement=800)


parsed_data = get_output_from_qchem(txt_input,
                                    processors=4,
                                    use_mpi=True,
                                    parser=basic_optimization,
                                    force_recalculation=False)


#parsed_data = basic_optimization(data)
opt_molecule = parsed_data['optimized_molecule']

print(opt_molecule)

print('calculate frequencies')
txt_input = create_qchem_input(opt_molecule,
                               jobtype='freq',
                               method='b3lyp',
                               basis='cc-pVDZ',)

parsed_data = get_output_from_qchem(txt_input,
                                    processors=4,
                                    use_mpi=True,
                                    force_recalculation=False,
                                    parser=basic_frequencies)

energy_i = parsed_data['energy']

f = open('{}.xyz'.format(args.filename), 'w')
f2 = open(args.filename, 'w')

# print data info

f2.write('OPTIMIZED STRUCTURE\n')
f2.write(opt_molecule)
f2.write('\n\n')
f2.write('DISPLACED STRUCTURES\n')

num_freq = len(parsed_data['frequencies'])

print('frequencies:', parsed_data['frequencies'])

for mode in range(num_freq):

    f2.write('----------------------------------\n')
    freq = parsed_data['frequencies'][mode]
    red_mass = parsed_data['red_mass'][mode]
    force_constants = parsed_data['force_constants'][mode]

    f2.write('mode:                      {}\n'.format(mode+1))
    f2.write('frequency (cm-1):          {}\n'.format(freq))
    f2.write('force constant (mdyne/A):  {}\n'.format(force_constants))

    if freq < 10:
        continue

    # prediction average displacement
    kb = 0.001987204  # kcal/molK
    max_a = np.sqrt(args.t * kb / (force_constants * 143.9325))
    f2.write('Average Kinetic at {:4} K: {:8.3f} Kcal/mol\n'.format(args.t, 0.5 * args.t * kb))

    data_for_plot = []
    for d in np.arange(-max_a, max_a+(max_a/11), max_a/10):
        mode_coor = parsed_data['coordinates'][mode]
        disp_coor = np.array(opt_molecule.get_coordinates()) + d * np.array(mode_coor)
        rmsd = get_rmsd(disp_coor, opt_molecule.get_coordinates())
        f2.write(''.format(rmsd))
        mol = Structure(coordinates=disp_coor,
                        symbols=symbols)

        hartree_to_kcalmol = 627.509474
        # energy2 = 0.5 * freq**2 * red_mass * d**2 * 8.516358951e-5  # kcal/mol
        energy2 = 0.5 * force_constants * d ** 2 * 143.9325  # kcal/mol

        if args.show_plot:
            txt_input = create_qchem_input(mol,
                                           jobtype='freq',
                                           method='b3lyp',
                                           basis='cc-pVDZ')

            parsed_data = get_output_from_qchem(txt_input, processors=4,
                                                force_recalculation=False,
                                                parser=basic_frequencies)
            energy = parsed_data['energy']

            data_for_plot.append([d, (energy - energy_i) * hartree_to_kcalmol, energy2])  # kcal/mol

        f.write(mol.get_xyz(title='{:8.5f} kcal/mol  rmsd: {:10.8f}\n'.format(energy2, rmsd)))
        f2.write(mol.get_xyz(title='{:8.5f} kcal/mol  rmsd: {:10.8f}\n'.format(energy2, rmsd)))

    if args.show_plot:
        import matplotlib.pyplot as plt
        plt.plot(np.array(data_for_plot)[:, 0], np.array(data_for_plot)[:, 1:])
        plt.show()
f.close()
f2.close()



from pyqchem.qchem_core import get_output_from_qchem, redefine_calculation_data_filename
from pyqchem.qc_input import QchemInput
from pyqchem.structure import Structure
from pyqchem.parsers.parser_rasci import parser_rasci as parser_rasci
from pyqchem.basis import get_basis_from_ccRepo, trucate_basis, basis_to_txt
import numpy as np
from pyqchem.errors import OutputError


redefine_calculation_data_filename('test_soc3.pkl')

as_c = [[[3, 1], 4, 1]]
atom_c = Structure(coordinates=[[0.0, 0.0, 0.0]],
                   symbols=['C'],
                   charge=-1,
                   multiplicity=4,
                   name='C')

as_o = [[[4, 2], 4, 1]]
atom_o = Structure(coordinates=[[0.0, 0.0, 0.0]],
                   symbols=['O'],
                   charge=-2,
                   multiplicity=1,
                   name='O')

as_si = [[[3, 1], 4, 5], [[6, 4], 7, 2]]
atom_si = Structure(coordinates=[[0.0, 0.0, 0.0]],
                    symbols=['Si'],
                    charge=-1,
                    multiplicity=4,
                    name='Si')

as_s = [[[3, 1], 4, 5], [[7, 5], 7, 2], [[8, 6], 8, 1]]
atom_s = Structure(coordinates=[[0.0, 0.0, 0.0]],
                   symbols=['S'],
                   charge=-2,
                   multiplicity=1,
                   name='S')

as_f = [[[4, 3], 4, 1]]
atom_f = Structure(coordinates=[[0.0, 0.0, 0.0]],
                   symbols=['F'],
                   charge=-1,
                   multiplicity=1,
                   name='F')

as_cl = [[[4, 3], 4, 5], [[7, 6], 7, 2], [[8, 7], 8, 1]]
atom_cl = Structure(coordinates=[[0.0, 0.0, 0.0]],
                    symbols=['Cl'],
                    charge=-1,
                    multiplicity=1,
                    name='Cl')


for atom, active_space_list in [(atom_s, as_s)]:#, (atom_si, as_si), (atom_c, as_c), (atom_o, as_o),  (atom_f, as_f), (atom_cl, as_cl)]:
    for basis_name in ['cc-pVDZ', 'cc-pCVTZ', 'cc-pCVQZ']:
        for active_space in active_space_list:
            basis_custom_repo = get_basis_from_ccRepo(atom, basis_name)

            basis_custom_repo = trucate_basis(basis_custom_repo,
                                              shells=['G', 'H', 'I', 'J', 'K'])

            qc_input = QchemInput(atom,
                                  jobtype='sp',
                                  exchange='hf',
                                  correlation='rasci',
                                  unrestricted='False',
                                  thresh=14,
                                  scf_convergence=8,
                                  max_cis_cycles=150,
                                  max_scf_cycles=100,
                                  basis=basis_custom_repo,
                                  # ras_elec=active_space[0],
                                  ras_elec_alpha=active_space[0][0],
                                  ras_elec_beta=active_space[0][1],
                                  ras_act=active_space[1],
                                  ras_occ=active_space[2],
                                  ras_spin_mult=0,
                                  ras_roots=8,  # calculate 5 states
                                  calc_soc=1,
                                  # n_frozen_core=0,
                                  set_iter=1000,
                                  mem_total=15000,
                                  mem_static=900
                                  )

            try:
                output = get_output_from_qchem(qc_input,
                                               processors=4,
                                               force_recalculation=False,
                                               parser=parser_rasci,
                                               store_full_output=True
                                               )
            except OutputError as e:
                print('---------------------------------------------')
                print('Atom: {}'.format(atom.name))
                print('basis: {}'.format(basis_name))
                print('Active space ([alpha, beta], act, occ): {}'.format(active_space))
                print(e.error_lines)
                print('calculation_failed')
                continue

            print('\n---------------------------------------------')
            print('Atom: {}'.format(atom.name))
            print('Basis: {}'.format(basis_name))
            print('Active space (ele, act, occ): {}'.format(active_space))
            print('---------------------------------------------')

            for i, state in enumerate(output['excited_states']):
                print('Energy state {} ({}):  {: 18.12f} au'.format(i+1, state['multiplicity'], state['total_energy']))
                print(' Alpha  Beta   Amplitude')
                for j, conf in enumerate(state['configurations']):
                    print('  {}  {} {:8.3f}'.format(conf['alpha'], conf['beta'], conf['amplitude']))

            print('\ngamma_tot')
            print('   ' + ''.join(['{:^18}'.format(n) for n in range(1, 9)]))
            for i in range(1, 9):
                line = '{:3}'.format(i)
                for j in range(1, 9):
                    try:
                        line += '{:18.12f}'.format(output['interstate_properties'][(i, j)]['gamma_total'])
                    except KeyError:
                        line += '        -         '
                print(line)

            print('\nsoc_1e (cm-1) [imaginary part]')
            print('   ' + ''.join(['{:^18}'.format(n) for n in range(1, 9)]))
            for i in range(1, 9):
                line = '{:3}'.format(i)
                for j in range(1, 9):
                    try:
                        line += '{:18.12f}'.format(np.array(output['interstate_properties'][(i, j)]['1e_soc_mat'])[0, 0].imag)
                    except KeyError:
                        line += '        -         '
                print(line)

            print('\nsoc_2e (cm-1) [imaginary part]')
            print('   ' + ''.join(['{:^18}'.format(n) for n in range(1, 9)]))
            for i in range(1, 9):
                line = '{:3}'.format(i)
                for j in range(1, 9):
                    try:
                        line += '{:18.12f}'.format(np.array(output['interstate_properties'][(i, j)]['2e_soc_mat'])[0, 0].imag)
                    except KeyError:
                        line += '        -         '
                print(line)

            print('\nsoc_tot (cm-1) [imaginary part]')
            print('   ' + ''.join(['{:^18}'.format(n) for n in range(1, 9)]))
            for i in range(1, 9):
                line = '{:3}'.format(i)
                for j in range(1, 9):
                    try:
                        line += '{:18.12f}'.format(np.array(output['interstate_properties'][(i, j)]['total_soc_mat'])[0, 0].imag)
                    except KeyError:
                        line += '        -         '
                print(line)

            print('Frozen occupied: {} / virtual: {}'.format(output['rasci_dimensions']['frozen_occupied'],
                                                             output['rasci_dimensions']['frozen_virtual']))

            print('\nSOCC (cm-1)')
            print('   ' + ''.join(['{:^18}'.format(n) for n in range(1, 9)]))
            for i in range(1, 9):
                line = '{:3}'.format(i)
                for j in range(1, 9):
                    try:
                        line += '{:18.12f}'.format(output['interstate_properties'][(i, j)]['mf_socc'])
                    except KeyError:
                        line += '        -         '
                print(line)

            print('Frozen occupied: {} / virtual: {}'.format(output['rasci_dimensions']['frozen_occupied'],
                                                             output['rasci_dimensions']['frozen_virtual']))



from pyqchem.qchem_core import get_output_from_qchem
from pyqchem.qc_input import QchemInput
from pyqchem.structure import Structure
from pyqchem.parsers.basic import basic_parser_qchem
from pyqchem.basis import get_basis_from_ccRepo
from pyqchem.basis import get_basis_from_BSE


# create molecule
molecule = Structure(coordinates=[[0.0, 0.0, 0.0000],
                                  [0.0, 0.0, 1.5811]],
                     symbols=['Se', 'H'],
                     charge=-1,
                     multiplicity=1)

# Standard input using 6-31G basis
qc_input = QchemInput(molecule,
                      jobtype='sp',
                      exchange='hf',
                      basis='6-31G')

# print input
print(qc_input.get_txt())

output, electronic_structure = get_output_from_qchem(qc_input,
                                                     processors=4,
                                                     force_recalculation=False,
                                                     read_fchk=True,
                                                     parser=basic_parser_qchem,
                                                     store_full_output=False)


print('scf_energy 6-31G: ', output['scf_energy'])

basis_custom = electronic_structure['basis']

# Standard input using a custom basis obtained from previous calculation
qc_input = QchemInput(molecule,
                      jobtype='sp',
                      exchange='hf',
                      basis=basis_custom)

output = get_output_from_qchem(qc_input,
                               processors=4,
                               force_recalculation=False,
                               parser=basic_parser_qchem
                               )

print('scf_energy (custom basis: 6-31G): ', output['scf_energy'])

# Get custom basis from ccRepo online repository
basis_custom_repo = get_basis_from_ccRepo(molecule, 'cc-pVTZ')

qc_input = QchemInput(molecule,
                      jobtype='sp',
                      exchange='hf',
                      basis=basis_custom_repo)


output = get_output_from_qchem(qc_input,
                               processors=4,
                               force_recalculation=False,
                               parser=basic_parser_qchem
                               )

print('scf_energy (custom basis: cc-pVTZ): ', output['scf_energy'])

basis_custom_repo = get_basis_from_BSE(molecule, '4zapa-nr', if_missing='6-31g')

qc_input = QchemInput(molecule,
                      jobtype='sp',
                      exchange='hf',
                      basis=basis_custom_repo)


output = get_output_from_qchem(qc_input,
                               processors=4,
                               force_recalculation=False,
                               parser=basic_parser_qchem
                               )

print('scf_energy (custom basis: coemd): ', output['scf_energy'])



from pyqchem.qchem_core import get_output_from_qchem
from pyqchem.qc_input import QchemInput
from pyqchem.parsers.parser_optimization import basic_optimization
from pyqchem.parsers.parser_rasci import parser_rasci
from pyqchem.structure import Structure
from pyqchem.plots import plot_diabatization, plot_state
from pyqchem.utils import get_ratio_of_condition

import numpy as np
import matplotlib.pyplot as plt


# define monomer
coor_monomer = [[ 0.6695,  0.0000,  0.0000],
                [-0.6695,  0.0000,  0.0000],
                [ 1.2321,  0.9289,  0.0000],
                [ 1.2321, -0.9289,  0.0000],
                [-1.2321,  0.9289,  0.0000],
                [-1.2321, -0.9289,  0.0000]]

symbols_monomer = ['C', 'C', 'H', 'H', 'H', 'H']

monomer = Structure(coordinates=coor_monomer,
                    symbols=symbols_monomer,
                    charge=0,
                    multiplicity=1)

# optimization qchem input
qc_input = QchemInput(monomer,
                      jobtype='opt',
                      exchange='hf',
                      basis='sto-3g',
                      geom_opt_tol_gradient=300,
                      geom_opt_tol_energy=100,
                      geom_opt_coords=-1,
                      geom_opt_tol_displacement=1200)

parsed_data = get_output_from_qchem(qc_input,
                                    processors=4,
                                    parser=basic_optimization)

opt_monomer = parsed_data['optimized_molecule']

print('Optimized monomer structure')
print(opt_monomer)

# Build dimer from monomer
coor_monomer2 = np.array(opt_monomer.get_coordinates())
coor_monomer2[:, 2] += 4.0  # monomers separation

coordinates = opt_monomer.get_coordinates() + coor_monomer2.tolist()
symbols_dimer = symbols_monomer * 2

dimer = Structure(coordinates=coordinates,
                  symbols=symbols_dimer,
                  charge=0,
                  multiplicity=1)

print('Dimer structure')
print(dimer)

# sequential diabatization scheme (2 steps)
diabatization_scheme = [{'method': 'ER', # first step
                         'states': [1, 2, 3, 4]},  # in order with respect to selected states
                        {'method': 'DQ', # Note: This second step is not really needed in this case, just for demostration
                         'states': [3, 4],  # in energy order (low->high) with respect to diabatic states in previous step
                         'parameters': 0.0}]

# RASCI qchem input
qc_input = QchemInput(dimer,
                      jobtype='sp',
                      exchange='hf',
                      correlation='rasci',
                      basis='sto-3g',
                      ras_act=6,
                      ras_elec=4,
                      ras_spin_mult=1,  # singlets only
                      ras_roots=8,      # calculate 8 states
                      ras_do_hole=False,
                      ras_do_part=False,
                      ras_diabatization_states=[3, 4, 5, 6],  # adiabatic states selected for diabatization
                      ras_diabatization_scheme=diabatization_scheme,
                      set_iter=30)

# print(qc_input.get_txt())

parsed_data = get_output_from_qchem(qc_input,
                                    processors=4,
                                    force_recalculation=False,
                                    parser=parser_rasci
                                    )

# parsed_data = rasci_parser(parsed_data)
# print(parsed_data)

# print adiabatic states
print('Adiabatic states dimer\n--------------------')
for i, state in enumerate(parsed_data['excited_states']):
    print('\nState {}'.format(i+1))
    print('Transition DM: ', state['transition_moment'])
    print('Energy: ', state['excitation_energy'])
    print(' Alpha  Beta   Amplitude')
    for j, conf in enumerate(state['configurations']):
        print('  {}  {} {:8.3f}'.format(conf['alpha'], conf['beta'], conf['amplitude']))

# plot adiabatic states
for i, state in enumerate(parsed_data['excited_states']):
    plot_state(state, with_amplitude=True, orbital_range=[qc_input._ras_occ,
                                                          qc_input._ras_occ + qc_input._ras_act])
    plt.title('Adiabatic State {}'.format(i+1))

plt.show()


# Analysis of diabatic states to use in diabatization
print('\nAdiabatic states to use in diabatization (1e, max_jump 4)')
for i, state in enumerate(parsed_data['excited_states']):
    ratio = get_ratio_of_condition(state, n_electron=1, max_jump=4)
    mark = 'X' if ratio > 0.5 else ''
    print('State {}: {:4.3f}  {}'.format(i+1, ratio, mark))


# diabatization analysis
diabatization = parsed_data['diabatization']

print('\nRotation Matrix')
print(np.array(diabatization['rot_matrix']))

print('\nAdiabatic Matrix')
print(np.array(diabatization['adiabatic_matrix']))

print('\nDiabatic Matrix')
print(np.array(diabatization['diabatic_matrix']))


print('\nDiabatic states dimer\n--------------------')

for i, state in enumerate(diabatization['diabatic_states']):
    print('\nState {}'.format(i+1))
    print('Transition DM: ', state['transition_moment'])
    print('Energy: ', state['excitation_energy'])

plot_diabatization(diabatization['diabatic_states'], atoms_ranges=[dimer.get_number_of_atoms()/2,
                                                                   dimer.get_number_of_atoms()])


# Monomer adiabatic states (extra test)
qc_input = QchemInput(opt_monomer,
                      jobtype='sp',
                      exchange='hf',
                      correlation='rasci',
                      basis='sto-3g',
                      ras_act=6,
                      ras_elec=4,
                      ras_spin_mult=1,  # singlets only
                      ras_roots=4,      # calculate 8 states
                      ras_do_hole=False,
                      ras_do_part=False,
                      set_iter=30)

parsed_data = get_output_from_qchem(qc_input,
                                    processors=14,
                                    force_recalculation=False,
                                    parser=parser_rasci
                                    )

print('\nAdiabatic states monomer\n--------------------')
for i, state in enumerate(parsed_data['excited_states']):
    print('\nState {}'.format(i+1))
    print('Transition DM: ', state['transition_moment'])
    print('Energy: ', state['excitation_energy'])
    print(' Alpha  Beta   Amplitude')
    for j, conf in enumerate(state['configurations']):
        print('  {}  {} {:8.3f}'.format(conf['alpha'], conf['beta'], conf['amplitude']))



from pyqchem.qchem_core import get_output_from_qchem, create_qchem_input
from pyqchem.parsers.parser_frequencies import basic_frequencies
from pyqchem.structure import Structure


remote_data = {'hostname': '111.222.333.44', # the remote machine host
               'port': 22,                   # port for SSH connection
               'username': 'user',           # your username
               'password': 'thepassword',    # your password
               'allow_agent': False,
               'look_for_keys': False,
               'precommand': ['module load qchem']}  # some prepend lines you may need to load qchem

# define molecule
coordinates = [[0.0,  1.0,  0.0],
               [0.0,  0.0,  1.0],
               [0.0,  0.0, -1.0]]

symbols = ['O', 'H', 'H']

molecule = Structure(coordinates=coordinates,
                     symbols=symbols,
                     charge=0,
                     multiplicity=1)

print('Initial structure')
print(molecule)

# calculation
qc_input = create_qchem_input(molecule,
                              jobtype='freq',
                              exchange='hf',
                              basis='sto-3g')

out, electronic_structure = get_output_from_qchem(qc_input,
                                                  processors=4,
                                                  read_fchk=True,
                                                  force_recalculation=True,
                                                  remote=remote_data,  # Set remote data
                                                  parser=basic_frequencies
                                                  )

# Show output
print(out)

# Show Electronic structure data
print(electronic_structure)


import pickle
import numpy as np
import matplotlib.pyplot as plt
import matplotlib as mpl
from matplotlib.backends.backend_pdf import PdfPages
from mpl_toolkits.mplot3d import axes3d  # necessary !!
from pyqchem.order_states import get_order_states_list, correct_order_list

mpl.rcParams.update({'font.size': 13})

import argparse

# Argument parser
parser = argparse.ArgumentParser(description='Plot 3D data')
parser.add_argument('filename', metavar='filename', type=str,
                    help='filename for input')

parser.add_argument('--output_folder', metavar='distance', type=str, default='3d_plot/',
                    help='folder to store PDF plots')

parser.add_argument('--show_plots', action='store_true',
                   help='show plots while running')

parser.add_argument('--full', action='store_true',
                   help='apply periodicity (centered at 0,0)')

parser.add_argument('--interpolate', action='store_true',
                   help='interpolate missing points')


args = parser.parse_args()


def interpolate_data(points, data , y_range, z_range):
    from scipy.interpolate import griddata

    Y, Z = np.meshgrid(y_range, z_range)

    grid_z2 = griddata(points, data, (Y, Z), method='cubic')

    return grid_z2.T.flatten()



default_range = 0.4

def triplot2(datas, labels, y_range, z_range, wireframe=False, pdf=None, clabels=False,
            zlabel='Energy [eV]', zlevels=np.arange(-default_range, default_range + 0.025, 0.025),
             show_plot=True, colors=('b', 'r', 'y', 'm')):

    # from matplotlib.colors import LinearSegmentedColormap
    # from matplotlib.colors import BoundaryNorm
    # from matplotlib.ticker import MaxNLocator

    cmap = plt.get_cmap('PiYG')

    Y, Z = np.meshgrid(z_range, y_range)

    for iplot, data in enumerate(datas):
        plt.figure(iplot+1)

        plt.title(labels[iplot])
        #zlevels = np.arange(-default_range, default_range + 0.025, 0.025)
        CS = plt.contourf(Y, Z, np.array(data).reshape(len(y_range), len(z_range)), levels=zlevels, cmap=cmap)
        # print(CS.levels[::1])

        # force symmetry
        #dist = CS.levels[1] - CS.levels[0]
        #if np.abs(CS.levels[0]) < (CS.levels[-1]):
        #    zlevels = np.arange(-CS.levels[-1], CS.levels[-1], dist)
        #else:
        #    zlevels = np.arange(CS.levels[0], -CS.levels[0], dist)
        #CS = plt.contourf(Y, Z, np.array(data).reshape(len(y_range), len(z_range)), levels=zlevels, cmap=cmap)

        CS2 = plt.contour(CS, levels=CS.levels[::1], colors='black', linestyles='solid')
        if clabels:
            plt.clabel(CS2, inline=1, fontsize=10)

        plt.xlabel('X [Å]')
        plt.ylabel('Y [Å]')
        plt.xlim([-4, 4])
        plt.ylim([-4, 4])

        cbar = plt.figure(iplot+1).colorbar(CS)

        cbar.ax.set_ylabel(zlabel)

        if pdf is not None:
            pdf.savefig()

        if not show_plot:
            plt.close()

    fig = plt.figure()

    ax = fig.add_subplot(111, projection='3d')

    fake2Dline = []
    for iplot, data in enumerate(datas):
        if wireframe:
            ax.plot_wireframe(Y, Z, np.array(data).reshape(len(y_range), len(z_range)), color=colors[iplot])
        else:
            ax.plot_surface(Y, Z, np.array(data).reshape(len(y_range), len(z_range)), color=colors[iplot])
        fake2Dline.append(mpl.lines.Line2D([0], [0], linestyle="none", c=colors[iplot], marker='o'))
    ax.legend(fake2Dline, labels, numpoints=1)
    ax.set_xlabel('X [Å]')
    ax.set_ylabel('Y [Å]')
    ax.set_zlabel(zlabel)

    if pdf is not None:
        pdf.savefig(fig)

    if show_plot:
        plt.show()
    else:
        plt.close()



def triplot3(datas, labels, y_range, z_range, wireframe=False, pdf=None, clabels=False,
            zlabel='Energy [eV]', zlevels=np.arange(-default_range, default_range + 0.025, 0.025),
             show_plot=True, colors=('b', 'r', 'y', 'm')):

    # from matplotlib.colors import LinearSegmentedColormap
    # from matplotlib.colors import BoundaryNorm
    # from matplotlib.ticker import MaxNLocator

    cmap = plt.get_cmap('PiYG')

    Y, Z = np.meshgrid(z_range, y_range)

    for iplot, data in enumerate(datas):
        plt.figure(iplot+1)

        plt.title(labels[iplot])
        #zlevels = np.arange(-default_range, default_range + 0.025, 0.025)
        CS = plt.contourf(Y, Z, np.array(data).reshape(len(y_range), len(z_range)), levels=zlevels, cmap=cmap)
        # print(CS.levels[::1])

        # force symmetry
        #dist = CS.levels[1] - CS.levels[0]
        #if np.abs(CS.levels[0]) < (CS.levels[-1]):
        #    zlevels = np.arange(-CS.levels[-1], CS.levels[-1], dist)
        #else:
        #    zlevels = np.arange(CS.levels[0], -CS.levels[0], dist)
        #CS = plt.contourf(Y, Z, np.array(data).reshape(len(y_range), len(z_range)), levels=zlevels, cmap=cmap)

        CS2 = plt.contour(CS, levels=CS.levels[::1], colors='black', linestyles='solid')
        if clabels:
            plt.clabel(CS2, inline=1, fontsize=10)

        plt.xlabel('X [Å]')
        plt.ylabel('Y [Å]')
        plt.xlim([-4, 4])
        plt.ylim([-4, 4])

        cbar = plt.figure(iplot+1).colorbar(CS)

        cbar.ax.set_ylabel(zlabel)

        if pdf is not None:
            pdf.savefig()

        if not show_plot:
            plt.close()

    fig = plt.figure()

    ax = fig.add_subplot(111, projection='3d')

    fake2Dline = []
    if wireframe:
        ax.plot_wireframe(Y, Z, np.array(datas[0]).reshape(len(y_range), len(z_range)), color=colors[0])
    else:
        ax.plot_surface(Y, Z, np.array(datas[0]).reshape(len(y_range), len(z_range)), color=colors[0])
    fake2Dline.append(mpl.lines.Line2D([0], [0], linestyle="none", c=colors[0], marker='o'))
    ax.set_xlabel('X [Å]')
    ax.set_ylabel('Y [Å]')
    ax.set_zlabel(zlabel)


    fig2 = plt.figure()

    ax2 = fig2.add_subplot(111, projection='3d')

    fake2Dline = []
    if wireframe:
        ax2.plot_wireframe(Y, Z, np.array(datas[1]).reshape(len(y_range), len(z_range)), color=colors[1])
    else:
        ax2.plot_surface(Y, Z, np.array(datas[1]).reshape(len(y_range), len(z_range)), color=colors[1])
    fake2Dline.append(mpl.lines.Line2D([0], [0], linestyle="none", c=colors[1], marker='o'))
    ax2.set_xlabel('X [Å]')
    ax2.set_ylabel('Y [Å]')
    ax2.set_zlabel(zlabel)



    if pdf is not None:
        pdf.savefig(fig)
        pdf.savefig(fig2)

    if show_plot:
        plt.show()
    else:
        plt.close()

def biplot_interpolated(data1, data2, label1, label2, y_range, z_range, pdf=None,
                        zlabel='Energy [eV]', zrange=(-default_range, default_range), zp=0.025, show_plot=True):

    from scipy import interpolate

    Y, Z = np.meshgrid(z_range, y_range)

    print('----------------')
    print(Y.shape)
    print(len(Y), len(data1))

    #f1 = interpolate.interp2d(Y, Z, data1, kind='cubic')
    f1 = interpolate.interp2d(Y, Z, data1, kind='linear')
    f2 = interpolate.interp2d(Y, Z, data2, kind='linear')

    x = np.arange(-4, 4, zp)

    plt.figure(3)

    plt.xlim([-4, 4])
    plt.ylim([zrange[0], zrange[1]])
    plt.xlabel('X [Å]')
    plt.ylabel(zlabel)

    plt.plot(x, f1(x, 0), label=label1)
    plt.plot(x, f2(x, 0), label=label2)
    plt.legend()

    if pdf is not None:
        pdf.savefig()

    if show_plot:
        plt.show()
    else:
        plt.close()


def biplot(data1, data2, label1, label2, y_range, z_range, pdf=None,
           zlabel='Energy [eV]', zrange=(-default_range, default_range), title=None, show_plot=True,
           direction=0, colors=[None, None]):

    plt.figure(3)
    plt.gcf().subplots_adjust(left=0.15)

    if direction == 0:
        data1 = np.array(data1).reshape([len(y_range), len(z_range)])[len(z_range)//2]
        data2 = np.array(data2).reshape([len(y_range), len(z_range)])[len(z_range)//2]
        plt.xlabel('X [Å]')

    if direction == 1:
        data1 = np.array(data1).reshape([len(y_range), len(z_range)])[:,len(y_range)//2]
        data2 = np.array(data2).reshape([len(y_range), len(z_range)])[:,len(y_range)//2]
        plt.xlabel('Y [Å]')

    plt.title(title)
    plt.xlim([-4, 4])
    if zrange is not None:
        plt.ylim([zrange[0], zrange[1]])
    plt.ylabel(zlabel)

    plt.plot(z_range, data1, label=label1, color=colors[0])
    plt.plot(z_range, data2, label=label2, color=colors[1])
    plt.legend()

    if pdf is not None:
        pdf.savefig()

    if show_plot:
        plt.show()
    else:
        plt.close()


def multibiplot(data, labels, y_range, z_range, pdf=None,
                zlabel='Energy [eV]', zrange=(-default_range, default_range), title=None, show_plot=True,
                direction=0):

    plt.figure(3)

    if direction == 0:
        for i, dat in enumerate(data):
            data[i] = np.array(dat).reshape([len(y_range), len(z_range)])[len(z_range)//2]
            #data[i] = np.array(dat).reshape([len(y_range), len(z_range)])[len(z_range)//2]
            plt.xlabel('X [Å]')

    if direction == 1:
        for i, dat in enumerate(data):

            data[i] = np.array(dat).reshape([len(y_range), len(z_range)])[:,len(y_range)//2]
            # data2 = np.array(data2).reshape([len(y_range), len(z_range)])[:,len(y_range)//2]
            plt.xlabel('Y [Å]')

    plt.title(title)
    plt.xlim([-4, 4])
    if zrange is not None:
        plt.ylim([zrange[0], zrange[1]])
    plt.ylabel(zlabel)
    for dat, l in zip(data, labels):
        plt.plot(z_range, dat, label=l)
        # plt.plot(z_range, data2, label=label2)
    plt.legend()

    if pdf is not None:
        pdf.savefig()

    if show_plot:
        plt.show()
    else:
        plt.close()


def multibiplot_2axis(data, labels, data2, labels2, y_range, z_range, pdf=None,
                      zlabel='Energy [eV]', zlabel2=' ', zrange=(-default_range, default_range),
                      zrange2=(-default_range, default_range), title=None, show_plot=True, direction=0,
                      baseline=None):

    #plt.figure(3)
    f, ax1 = plt.subplots()
    ax2 = ax1.twinx()

    if direction == 0:
        ax1.set_xlabel('X [Å]')
        for i, dat in enumerate(data):
            data[i] = np.array(dat).reshape([len(y_range), len(z_range)])[len(z_range)//2]
        for i, dat in enumerate(data2):
            data2[i] = np.array(dat).reshape([len(y_range), len(z_range)])[len(z_range)//2]

    if direction == 1:
        ax1.set_xlabel('Y [Å]')
        for i, dat in enumerate(data):
            data[i] = np.array(dat).reshape([len(y_range), len(z_range)])[:,len(y_range)//2]
            # data2 = np.array(data2).reshape([len(y_range), len(z_range)])[:,len(y_range)//2]
        for i, dat in enumerate(data2):
            data2[i] = np.array(dat).reshape([len(y_range), len(z_range)])[:,len(y_range)//2]

    plt.title(title)
    ax1.set_xlim([-4, 4])
    if zrange is not None:
        ax1.set_ylim([zrange[0], zrange[1]])
        ax1.set_ylabel(zlabel)
    if zrange2 is not None:
        ax2.set_ylim([zrange2[0], zrange2[1]])
        ax2.set_ylabel(zlabel2)

    lns1 = []
    for dat, l in zip(data, labels):
        lns1.append(ax1.plot(z_range, dat, label=l))
        # plt.plot(z_range, data2, label=label2)

    lns2 = []
    for dat, l in zip(data2, labels2):
        lns2.append(ax2.plot(z_range, dat, label=l, color='grey'))

    lns = [j for i in lns1 for j in i] + [j for i in lns2 for j in i]
    labs = [l.get_label() for l in lns]
    plt.legend(lns, labs, loc=0)

    if baseline is not None:
        ax1.axhline(baseline, color='k', linestyle=':')

    if pdf is not None:
        pdf.savefig()

    if show_plot:
        plt.show()
    else:
        plt.close()

def get_data(total_data, y_range, z_range, points,
             type=None, property=None, interpolate=False, average=False, states_orders=None):

    try:
        nval = len(total_data[0][type][property])
    except TypeError:
        nval = 1

    data_list = []
    for i in range(nval):
        if average:
            data_list.append([np.average(data[type][property]) for data in total_data])
        else:
            data_list.append(np.array([data[type][property][i] for data in total_data]))

    if states_orders is not None:
        data_list = correct_order_list(data_list, states_orders)

    if interpolate:
        for i in range(nval):
            data_list[i] = interpolate_data(points, data_list[i], y_range, z_range)

    return data_list


#############################
folder = args.output_folder
#############################

with open(args.filename, 'rb') as input:
    calculation_data = pickle.load(input)
    print('Loaded data from: {}'.format(args.filename))

do_full = args.full
interpolate = args.interpolate

for slide_y in calculation_data['range_y']:
    for slide_z in calculation_data['range_z']:
        print(slide_y, slide_z)
        if '{}_{}'.format(slide_y, slide_z) in calculation_data:
            print(calculation_data['{}_{}'.format(slide_y, slide_z)])
            data_i = calculation_data['{}_{}'.format(slide_y, slide_z)]
            print(data_i['states_info'])
            if do_full:

                calculation_data.update({'{}_{}'.format(-slide_y, slide_z): data_i,
                                         '{}_{}'.format(slide_y, -slide_z): data_i,
                                         '{}_{}'.format(-slide_y, -slide_z): data_i})


if do_full:
    y_range = np.unique((-np.array(calculation_data['range_y'])).tolist() + calculation_data['range_y'])
    z_range = np.unique((-np.array(calculation_data['range_z'])).tolist() + calculation_data['range_z'])
else:
    y_range = calculation_data['range_y']
    z_range = calculation_data['range_z']

points = []
total_data = []
full_range = []
i = 0
for slide_y in y_range:
    for slide_z in z_range:
        print('{}_{}'.format(slide_y, slide_z))
        if '{}_{}'.format(slide_y, slide_z) in calculation_data:
            data = calculation_data['{}_{}'.format(slide_y, slide_z)]
            total_data.append(data)
            full_range.append(i)
            points.append([slide_y, slide_z])
            i += 1

# define order states
states_orders = [get_order_states_list(data['states_info']) for data in total_data]
# states_orders = [get_order_states_list(data['states_info'], eps_energy=0) for data in total_data]


########################  D_i  ######################

data_1 = [np.average(data['diabatic_energies']['E_LE']) for data in total_data]
data_2 = [np.average(data['diabatic_energies']['E_CT']) for data in total_data]

if interpolate:
    data_1 = interpolate_data(points, data_1, y_range, z_range)
    data_2 = interpolate_data(points, data_2, y_range, z_range)

e_le = np.array(data_1)
e_ct = np.array(data_2)

with PdfPages(folder + 'LE_CT.pdf') as pdf:
    triplot2([e_le, e_ct], ['$E_{LE}$', '$E_{CT}$'], y_range, z_range, pdf=pdf, wireframe=True,
             show_plot=args.show_plots, zlevels=None)
    biplot(e_le, e_ct, '$E_{LE}$', '$E_{CT}$', y_range, z_range, show_plot=args.show_plots,
           pdf=pdf, direction=0, zrange=[8, 14])
    biplot(e_le, e_ct, '$E_{LE}$', '$E_{CT}$', y_range, z_range, show_plot=args.show_plots,
           pdf=pdf, direction=1, zrange=[8, 14])


de_le, = get_data(total_data, y_range, z_range, points,
                  type='diabatic_energies', property='dE_LE',
                  interpolate=interpolate, average=True)

de_ct, = get_data(total_data, y_range, z_range, points,
                  type='diabatic_energies', property='dE_CT',
                  interpolate=interpolate, average=True)

d2le = get_data(total_data, y_range, z_range, points,
                type='asymmetries', property='delta2_le',
                interpolate=interpolate, average=False,
                states_orders=states_orders)

d2ct = get_data(total_data, y_range, z_range, points,
                type='asymmetries', property='delta2_ct',
                interpolate=interpolate, average=False,
                states_orders=states_orders)

lmb2 = get_data(total_data, y_range, z_range, points,
                type='asymmetries', property='lambda2',
                interpolate=interpolate, average=False,
                states_orders=states_orders)


di_1 = lmb2[0]*(e_ct - e_le) + d2le[0]*de_le + d2ct[0]*de_ct
di_2 = lmb2[1]*(e_ct - e_le) + d2le[1]*de_le + d2ct[1]*de_ct


with PdfPages(folder + 'delta.pdf') as pdf:
    triplot2([di_1, di_2], ['$\Delta_1$', '$\Delta_2$'], y_range, z_range, pdf=pdf, wireframe=True,
             show_plot=args.show_plots, zlevels=np.arange(-0.4, 0.4 + 0.02, 0.02))
    biplot(di_1, di_2, '$\Delta_1$', '$\Delta_2$', y_range, z_range, show_plot=args.show_plots,
           pdf=pdf, direction=0, zrange=[0.0, 0.35])
    biplot(di_1, di_2, '$\Delta_1$', '$\Delta_2$', y_range, z_range, show_plot=args.show_plots,
           pdf=pdf, direction=1, zrange=[0.0, 0.35])


########################  omega_DC  ######################

w_dc = get_data(total_data, y_range, z_range, points,
                type='diabatic_contributions', property='W_DC',
                interpolate=interpolate, average=False,
                states_orders=states_orders)

o_dc_1 = np.sqrt((1 - lmb2[0])**2 - d2le[0]**2) * w_dc[0]
o_dc_2 = np.sqrt((1 - lmb2[1])**2 - d2le[1]**2) * w_dc[1]

with PdfPages(folder + 'omega_dc.pdf') as pdf:
    triplot2([o_dc_1, o_dc_2], ['$\Omega_{DC}^{(1)}$', '$\Omega_{DC}^{(2)}$'], y_range, z_range, pdf=pdf, wireframe=True,
             show_plot=args.show_plots, zlevels=np.arange(-0.4, 0.4 + 0.05, 0.05))
    biplot(o_dc_1, o_dc_2, '$\Omega_{DC}^{(1)}$', '$\Omega_{DC}^{(2)}$', y_range, z_range,
           show_plot=args.show_plots, pdf=pdf, direction=0)
    biplot(o_dc_1, o_dc_2, '$\Omega_{DC}^{(1)}$', '$\Omega_{DC}^{(2)}$', y_range, z_range,
           show_plot=args.show_plots, pdf=pdf, direction=1)


########################  omega_CT  ######################

w_ct = get_data(total_data, y_range, z_range, points,
                type='diabatic_contributions', property='W_CT',
                interpolate=interpolate, average=False,
                states_orders=states_orders)

o_ct_1 = np.sqrt(lmb2[0]**2 - d2ct[0]**2) * w_ct[0]
o_ct_2 = np.sqrt(lmb2[1]**2 - d2ct[1]**2) * w_ct[1]

with PdfPages(folder + 'omega_ct.pdf') as pdf:
    triplot2([o_ct_1, o_ct_2], ['$\Omega_{CT}^{(1)}$', '$\Omega_{CT}^{(2)}$'], y_range, z_range, pdf=pdf, wireframe=True,
             show_plot=args.show_plots, zlevels=np.arange(-0.02, 0.02 + 0.005, 0.005))
    biplot(o_ct_1, o_ct_2, '$\Omega_{CT}^{(1)}$', '$\Omega_{CT}^{(2)}$', y_range, z_range,
           show_plot=args.show_plots, pdf=pdf, direction=0)
    biplot(o_ct_1, o_ct_2, '$\Omega_{CT}^{(1)}$', '$\Omega_{CT}^{(2)}$', y_range, z_range,
           show_plot=args.show_plots, pdf=pdf, direction=1)


########################  omega_h  ######################

o_h = get_data(total_data, y_range, z_range, points,
               type='diabatic_contributions', property='Omega_h',
               interpolate=interpolate, average=False,
               states_orders=states_orders)

w_h = get_data(total_data, y_range, z_range, points,
               type='diabatic_contributions', property='W_h',
               interpolate=interpolate, average=False,
               states_orders=states_orders)

oh_1 = 2*np.sqrt(lmb2[0] * (1-lmb2[0]) + lmb2[0]*d2le[0] + (1-lmb2[0])*d2ct[0] + d2le[0]*d2ct[0]) * w_h[0]
oh_2 = 2*np.sqrt(lmb2[1] * (1-lmb2[1]) + lmb2[1]*d2le[1] + (1-lmb2[1])*d2ct[1] + d2le[1]*d2ct[1]) * w_h[1]


with PdfPages(folder + 'omega_h.pdf') as pdf:
    triplot2([o_h[0], o_h[1]], ['$\Omega_{h}^{(1)}$', '$\Omega_{h}^{(2)}$'], y_range, z_range, pdf=pdf, wireframe=True,
             show_plot=args.show_plots, zlevels=None)
    biplot(o_h[0], o_h[1], '$\Omega_{h}^{(1)}$', '$\Omega_{h}^{(2)}$', y_range, z_range,
           show_plot=args.show_plots, pdf=pdf, direction=0)
    biplot(o_h[0], o_h[1], '$\Omega_{h}^{(1)}$', '$\Omega_{h}^{(2)}$', y_range, z_range,
           show_plot=args.show_plots, pdf=pdf, direction=1)


with PdfPages(folder + 'omega_hb.pdf') as pdf:
    triplot2([oh_1, oh_2], ['$\Omega_{h}^{(1)}$', '$\Omega_{h}^{(2)}$'], y_range, z_range, pdf=pdf, wireframe=True,
             show_plot=args.show_plots, zlevels=None)
    biplot(oh_1, oh_2, '$\Omega_{h}^{(1)}$', '$\Omega_{h}^{(2)}$', y_range, z_range,
           show_plot=args.show_plots, pdf=pdf, direction=0)
    biplot(oh_1, oh_2, '$\Omega_{h}^{(1)}$', '$\Omega_{h}^{(2)}$', y_range, z_range,
           show_plot=args.show_plots, pdf=pdf, direction=1)


########################  omega_e  ######################

o_e = get_data(total_data, y_range, z_range, points,
               type='diabatic_contributions', property='Omega_e',
               interpolate=interpolate, average=False,
               states_orders=states_orders)

w_e = get_data(total_data, y_range, z_range, points,
               type='diabatic_contributions', property='W_e',
               interpolate=interpolate, average=False,
               states_orders=states_orders)

oe_1 = 2 * np.sqrt(lmb2[0] * (1-lmb2[0]) + lmb2[0]*d2le[0] - (1-lmb2[0])*d2ct[0] - d2le[0]*d2ct[0]) * w_e[0]
oe_2 = 2 * np.sqrt(lmb2[1] * (1-lmb2[1]) + lmb2[1]*d2le[1] - (1-lmb2[1])*d2ct[1] - d2le[1]*d2ct[1]) * w_e[1]


with PdfPages(folder + 'omega_e.pdf') as pdf:
    triplot2([o_e[0], o_e[1]], ['$\Omega_{e}^{(1)}$', '$\Omega_{e}^{(2)}$'], y_range, z_range, pdf=pdf, wireframe=True,
             show_plot=args.show_plots, zlevels=None)
    biplot(o_e[0], o_e[1], '$\Omega_{e}^{(1)}$', '$\Omega_{e}^{(2)}$', y_range, z_range,
           show_plot=args.show_plots, pdf=pdf, direction=0)
    biplot(o_e[0], o_e[1], '$\Omega_{e}^{(1)}$', '$\Omega_{e}^{(2)}$', y_range, z_range,
           show_plot=args.show_plots, pdf=pdf, direction=1)


with PdfPages(folder + 'omega_eb.pdf') as pdf:
    triplot2([oe_1, oe_2], ['$\Omega_{e}^{(1)}$', '$\Omega_{e}^{(2)}$'], y_range, z_range, pdf=pdf, wireframe=True,
             show_plot=args.show_plots, zlevels=None)
    biplot(oe_1, oe_2, '$\Omega_{e}^{(1)}$', '$\Omega_{e}^{(2)}$', y_range, z_range,
           show_plot=args.show_plots, pdf=pdf, direction=0)
    biplot(oe_1, oe_2, '$\Omega_{e}^{(1)}$', '$\Omega_{e}^{(2)}$', y_range, z_range,
           show_plot=args.show_plots, pdf=pdf, direction=1)


######################### adiabatic energies ############################

osx_1 = o_h[0] + o_e[0]
osx_2 = o_h[1] + o_e[1]

with PdfPages(folder + 'omega_sx.pdf') as pdf:
    triplot2([osx_1, osx_2], ['$\Omega_{SX}^{(1)}$', '$\Omega_{SX}^{(2)}$'], y_range, z_range, pdf=pdf, wireframe=True,
             show_plot=args.show_plots, zlevels=np.arange(-0.12, 0.12 + 0.005, 0.005))
    biplot(osx_1, osx_2, '$\Omega_{SX}^{(1)}$', '$\Omega_{e}^{(2)}$', y_range, z_range,
           show_plot=args.show_plots, pdf=pdf, direction=0, zrange=[-1.5, 1.5])
    biplot(osx_1, osx_2, '$\Omega_{SX}^{(1)}$', '$\Omega_{e}^{(2)}$', y_range, z_range,
           show_plot=args.show_plots, pdf=pdf, direction=1, zrange=[-1.5, 1.5])


with PdfPages(folder + 'omega_sx2.pdf') as pdf:
    triplot2([osx_1, osx_2], ['$\Omega_{SX}^{(1)}$', '$\Omega_{SX}^{(2)}$'], y_range, z_range, pdf=pdf, wireframe=True,
             show_plot=args.show_plots, zlevels=np.arange(-0.01, 0.01 + 0.001, 0.001))
    biplot(osx_1, osx_2, '$\Omega_{SX}^{(1)}$', '$\Omega_{e}^{(2)}$', y_range, z_range,
           show_plot=args.show_plots, pdf=pdf, direction=0, zrange=[-0.2, 0.2])
    biplot(osx_1, osx_2, '$\Omega_{SX}^{(1)}$', '$\Omega_{e}^{(2)}$', y_range, z_range,
           show_plot=args.show_plots, pdf=pdf, direction=1, zrange=[-0.2, 0.2])


######################### figure 6 ############################


with PdfPages(folder + 'figure6.pdf') as pdf:
    multibiplot_2axis([o_dc_1, osx_1, o_ct_1, di_1], ['$\Omega^{(1)}_{DC}$', '$\Omega_{SX}^{(1)}$', '$\Omega_{CT}^{(1)}$', '$\Delta_1$'],
                      [lmb2[0]], ['$\lambda^2_1$'], y_range, z_range, pdf=pdf, zlabel2='$\lambda^2$',
                      zlabel='Energy [eV]', zrange=(-1.5, 1.5), zrange2=(0.0, 0.2), title='State 1',
                      show_plot=args.show_plots, direction=0, baseline=0.0)

    multibiplot_2axis([o_dc_1, osx_1, o_ct_1, di_1], ['$\Omega^{(1)}_{DC}$', '$\Omega_{SX}^{(1)}$', '$\Omega_{CT}^{(1)}$', '$\Delta_1$'],
                      [lmb2[0]], ['$\lambda^2_1$'], y_range, z_range, pdf=pdf, zlabel2='$\lambda^2$',
                      zlabel='Energy [eV]', zrange=(-1.5, 1.5), zrange2=(0.0, 0.2), title='State 1',
                      show_plot=args.show_plots, direction=1, baseline=0.0)

    multibiplot_2axis([o_dc_2, osx_2, o_ct_2, di_2], ['$\Omega^{(2)}_{DC}$', '$\Omega_{SX}^{(2)}$', '$\Omega_{CT}^{(2)}$', '$\Delta_2$'],
                      [lmb2[1]], ['$\lambda^2_2$'], y_range, z_range, pdf=pdf, zlabel2='$\lambda^2$',
                      zlabel='Energy [eV]', zrange=(-1.5, 1.5), zrange2=(0.0, 0.2), title='State 2',
                      show_plot=args.show_plots, direction=0, baseline=0.0)

    multibiplot_2axis([o_dc_2, osx_2, o_ct_2, di_2], ['$\Omega^{(2)}_{DC}$', '$\Omega_{SX}^{(2)}$', '$\Omega_{CT}^{(2)}$', '$\Delta_2$'],
                      [lmb2[1]], ['$\lambda^2_2$'], y_range, z_range, pdf=pdf, zlabel2='$\lambda^2$',
                      zlabel='Energy [eV]', zrange=(-1.5, 1.5), zrange2=(0.0, 0.2), title='State 2',
                      show_plot=args.show_plots, direction=1, baseline=0.0)


######################### adiabatic energies ############################

data_1 = [data['adiabatic_energies']['E_1'] for data in total_data]
data_2 = [data['adiabatic_energies']['E_2'] for data in total_data]
data_3 = [data['adiabatic_energies']['E_3'] for data in total_data]
data_4 = [data['adiabatic_energies']['E_4'] for data in total_data]

data_1, data_2, data_3, data_4 = correct_order_list([data_1, data_2, data_3, data_4], states_orders)

if interpolate:
    data_1 = interpolate_data(points, data_1, y_range, z_range)
    data_2 = interpolate_data(points, data_2, y_range, z_range)
    data_3 = interpolate_data(points, data_3, y_range, z_range)
    data_4 = interpolate_data(points, data_4, y_range, z_range)

e_1 = np.array(data_1)
e_2 = np.array(data_2)
e_3 = np.array(data_3)
e_4 = np.array(data_4)


with PdfPages(folder + 'figure4.pdf') as pdf:
    triplot3([e_1, e_2], ['$^1A_g$', '$^1B_{3u}$'], y_range, z_range, pdf=pdf, wireframe=True,
             show_plot=args.show_plots, zlevels=None, colors=['black', 'red'])
    biplot(e_1, e_2, '$^1A_g$', '$^1B_{3u}$', y_range, z_range,
           show_plot=args.show_plots, pdf=pdf, direction=0, zrange=[7.6, 9.2], colors=['black', 'red'])
    biplot(e_1, e_2, '$^1A_g$', '$^1B_{3u}$', y_range, z_range,
           show_plot=args.show_plots, pdf=pdf, direction=1, zrange=[7.6, 9.2], colors=['black', 'red'])

e_1b = e_le + di_1 + o_dc_1 + o_ct_1 + o_h[0] + o_e[0]
e_2b = e_le + di_2 + o_dc_2 + o_ct_2 + o_h[1] + o_e[1]

with PdfPages(folder + 'test.pdf') as pdf:
    #triplot2([e_1-e_1b, e_2-e_2b], ['$E^{(1)}$', '$E^{(2)}$'], y_range, z_range, pdf=pdf, wireframe=True,
    #         show_plot=args.show_plots, zlevels=None)
    biplot(e_1-e_1b, e_2-e_2b, '$E^{(1)}$', '$E^{(2)}$', y_range, z_range,
           show_plot=args.show_plots, pdf=pdf, direction=0)
    biplot(e_1-e_1b, e_2-e_2b, '$E^{(1)}$', '$E^{(2)}$', y_range, z_range,
           show_plot=args.show_plots, pdf=pdf, direction=1)


exit()

########################  omega 1,J,K  ######################

o_one_dc = get_data(total_data, y_range, z_range, points,
                    type='diabatic_contributions', property='Omega_One_dc',
                    interpolate=interpolate, average=False,
                    states_orders=states_orders)

o_j_dc = get_data(total_data, y_range, z_range, points,
                  type='diabatic_contributions', property='Omega_J_dc',
                  interpolate=interpolate, average=False,
                  states_orders=states_orders)
o_k_dc = get_data(total_data, y_range, z_range, points,
                  type='diabatic_contributions', property='Omega_K_dc',
                  interpolate=interpolate, average=False,
                  states_orders=states_orders)



with PdfPages(folder + 'omega_other.pdf') as pdf:
    #triplot2([o_dc_one_1, o_dc_one_2], ['$\Omega_{e}^{(1)}$', '$\Omega_{e}^{(2)}$'], y_range, z_range, pdf=pdf, wireframe=True,
    #          show_plot=args.show_plots, zlevels=None)
    multibiplot([o_one_dc[0], o_j_dc[0], o_k_dc[0]],
                ['$\Omega_{DC,1}^{(1)}$', '$\Omega_{DC,J}^{(1)}$', '$\Omega_{DC,K}^{(1)}$'], y_range, z_range,
                 show_plot=args.show_plots, pdf=pdf, direction=0)
    multibiplot([o_one_dc[0], o_j_dc[0], o_k_dc[0]],
                ['$\Omega_{DC,1}^{(1)}$', '$\Omega_{DC,J}^{(1)}$', '$\Omega_{DC,K}^{(1)}$'], y_range, z_range,
                 show_plot=args.show_plots, pdf=pdf, direction=1)


exit()



import numpy as np

from pyqchem.utils import get_plane
from pyqchem.qchem_core import get_output_from_qchem
from pyqchem.qc_input import QchemInput
from pyqchem.structure import Structure
from pyqchem.file_io import build_fchk
from pyqchem.parsers.parser_rasci import parser_rasci
from pyqchem.symmetry import get_state_symmetry


ethene = [[0.0,  0.0000,   0.65750],
          [0.0,  0.0000,  -0.65750],
          [0.0,  0.92281,  1.22792],
          [0.0, -0.92281,  1.22792],
          [0.0, -0.92281, -1.22792],
          [0.0,  0.92281, -1.22792]]

symbols = ['C', 'C', 'H', 'H', 'H', 'H']

# create molecule
molecule = Structure(coordinates=ethene,
                     symbols=symbols,
                     charge=0,
                     multiplicity=1)

# create Q-Chem input
qc_input = QchemInput(molecule,
                      jobtype='sp',
                      exchange='hf',
                      correlation='rasci',
                      ras_act=6,
                      ras_elec=6,
                      basis='sto-3g',
                      ras_roots=10,
                      ras_do_hole=False,
                      ras_do_part=False)

print(qc_input.get_txt())


# get data from Q-Chem calculation
output, electronic_structure = get_output_from_qchem(qc_input,
                                                     processors=4,
                                                     force_recalculation=False,
                                                     read_fchk=True,
                                                     parser=parser_rasci,
                                                     store_full_output=True)


# store original fchk info in file
open('test.fchk', 'w').write(build_fchk(electronic_structure))

# get plane from coordinates
coordinates_f1 = np.array(electronic_structure['structure'].get_coordinates())
center_f1, normal_f1 = get_plane(coordinates_f1)


symmetry_measures = get_state_symmetry(electronic_structure,
                                       output['excited_states'],
                                       center=center_f1,
                                       orientation=normal_f1,
                                       group='D2h',
                                       extra_print=False
                                       )

energies = [state['excitation_energy'] for state in output['excited_states']]

print('Symmetry of RASCI excited states\n--------------------------------')
for energy, state in zip(energies, symmetry_measures.items()):
    print('{:8} '.format(state[0]) + ' {:5} {:5.3f}'.format(*state[1]) + '  {:5.3f} eV'.format(energy))




from pyqchem.structure import Structure
from pyqchem.qchem_core import get_output_from_qchem, create_qchem_input
from pyqchem.parsers.basic import basic_parser_qchem
import numpy as np
import matplotlib.pyplot as plt


parameters = {'jobtype': 'sp',
              'exchange': 'hf',
              'correlation': 'rasci',
              'basis': '6-31G(d,p)',
              'ras_act': 2,
              'ras_elec': 2,
              'ras_spin_mult': 3,
              'ras_roots': 1,
              'ras_do_hole': True,
              'ras_sts_tm': True,
              # rasci sr-dft
              'ras_srdft': True,
              'ras_srdft_cor': 'srpw92',
              'ras_srdft_exc': 'srpbe',
              'ras_natorb': False,
              'ras_print': 0,
              'ras_srdft_spinpol': True,
              'set_iter': 60,
              'ras_srdft_damp': 0.5,
              'gui': 0}

energies = []
energies_es = []

atoms = ['C', 'O', 'Si']

omegas = [pow(10, i) for i in np.arange(1, 2, 0.25)]

atom_data = {}
for atom in atoms:
    data_omega = []

    for omega in omegas:
        try:
            molecule = Structure(coordinates=[[0.0, 0.0, 0.0]],
                                 symbols=[atom],
                                 charge=0)

            parameters.update({'ras_omega': int(omega)})

            gap_list = []
            for ispin in [True, False]:
                print(ispin)
                parameters.update({'ras_srdft_spinpol': ispin})

                # singlet
                parameters.update({'ras_spin_mult': 1})
                txt_input = create_qchem_input(molecule, **parameters)
                data_singlet = get_output_from_qchem(txt_input, processors=4, parser=basic_parser_qchem)

                # triplet
                parameters.update({'ras_spin_mult': 3})
                txt_input = create_qchem_input(molecule, **parameters)
                data_triplet = get_output_from_qchem(txt_input, processors=4, parser=basic_parser_qchem)

                gap = data_triplet['excited states'][0]['total energy'] - data_singlet['excited states'][0]['total energy']
                gap_list.append(gap)

        except:
            continue

        data_omega.append(gap_list[0]-gap_list[1])

    print(atom, data_omega)
    plt.plot(omegas, data_omega, label=atom)
    plt.xscale('log')
    plt.show()
    atom_data[atom] = data_omega

#plt.legend()
#plt.show()
print(atom_data)


import pickle
import numpy as np
import matplotlib.pyplot as plt
import matplotlib as mpl
from matplotlib.backends.backend_pdf import PdfPages
from mpl_toolkits.mplot3d import axes3d  # necessary !!
from pyqchem.order_states import get_order_states_list, correct_order_list

import argparse

# Argument parser
parser = argparse.ArgumentParser(description='Plot 3D data')
parser.add_argument('filename', metavar='filename', type=str,
                    help='filename for input')

parser.add_argument('--output_folder', metavar='distance', type=str, default='3d_plot/',
                    help='folder to store PDF plots')

parser.add_argument('--show_plots', action='store_true',
                   help='show plots while running')

parser.add_argument('--full', action='store_true',
                   help='apply periodicity (centered at 0,0)')

parser.add_argument('--interpolate', action='store_true',
                   help='interpolate missing points')


args = parser.parse_args()


def interpolate_data(points, data , y_range, z_range):
    from scipy.interpolate import griddata

    Y, Z = np.meshgrid(y_range, z_range)

    grid_z2 = griddata(points, data, (Y, Z), method='cubic')

    return grid_z2.T.flatten()


def triplot(data1, data2, label1, label2, y_range, z_range, wireframe=False, pdf=None, clabels=False,
            zlabel='Energy [eV]', zlevels=np.arange(-1.5, 1.5 + 0.025, 0.025), show_plot=True):

    # from matplotlib.colors import LinearSegmentedColormap
    # from matplotlib.colors import BoundaryNorm
    # from matplotlib.ticker import MaxNLocator

    cmap = plt.get_cmap('PiYG')

    Y, Z = np.meshgrid(z_range, y_range)

    plt.figure(1)

    plt.title(label1)
    CS = plt.contourf(Y, Z, np.array(data1).reshape(len(y_range), len(z_range)), levels=zlevels, cmap=cmap)
    CS2 = plt.contour(CS, levels=CS.levels[::1], colors='black')
    if clabels:
        plt.clabel(CS2, inline=1, fontsize=10)

    plt.xlabel('X [Å]')
    plt.ylabel('Y [Å]')
    plt.xlim([-4, 4])
    plt.ylim([-4, 4])

    cbar = plt.figure(1).colorbar(CS)

    cbar.ax.set_ylabel(zlabel)

    if pdf is not None:
        pdf.savefig()

    if not show_plot:
        plt.close()


    plt.figure(2)

    plt.title(label2)
    CS = plt.contourf(Y, Z, np.array(data2).reshape(len(y_range), len(z_range)), levels=zlevels, cmap=cmap)
    CS2 = plt.contour(CS, levels=CS.levels[::1], colors='black')

    if clabels:
        plt.clabel(CS2, inline=1, fontsize=10)

    plt.xlabel('X [Å]')
    plt.ylabel('Y [Å]')
    plt.xlim([-4, 4])
    plt.ylim([-4, 4])

    cbar = plt.figure(2).colorbar(CS)
    cbar.ax.set_ylabel(zlabel)

    if pdf is not None:
        pdf.savefig()

    if not show_plot:
        plt.close()

    fig = plt.figure()

    ax = fig.add_subplot(111, projection='3d')

    if wireframe:
        ax.plot_wireframe(Y, Z, np.array(data1).reshape(len(y_range), len(z_range)), color='b')
        ax.plot_wireframe(Y, Z, np.array(data2).reshape(len(y_range), len(z_range)), color='r')
    else:
        ax.plot_surface(Y, Z, np.array(data1).reshape(len(y_range), len(z_range)), color='b')
        ax.plot_surface(Y, Z, np.array(data2).reshape(len(y_range), len(z_range)), color='r')

    fake2Dline = mpl.lines.Line2D([0], [0], linestyle="none", c='b', marker='o')
    fake2Dline2 = mpl.lines.Line2D([0], [0], linestyle="none", c='r', marker='o')
    ax.legend([fake2Dline, fake2Dline2], [label1, label2], numpoints=1)
    ax.set_xlabel('X [Å]')
    ax.set_ylabel('Y [Å]')
    ax.set_zlabel(zlabel)

    if pdf is not None:
        pdf.savefig(fig)

    if show_plot:
        plt.show()
    else:
        plt.close()


def biplot_interpolated(data1, data2, label1, label2, y_range, z_range, pdf=None,
                        zlabel='Energy [eV]', zrange=(-1.5, 1.5), zp=0.025, show_plot=True):

    from scipy import interpolate

    Y, Z = np.meshgrid(z_range, y_range)

    print('----------------')
    print(Y.shape)
    print(len(Y), len(data1))

    #f1 = interpolate.interp2d(Y, Z, data1, kind='cubic')
    f1 = interpolate.interp2d(Y, Z, data1, kind='linear')
    f2 = interpolate.interp2d(Y, Z, data2, kind='linear')

    x = np.arange(-4, 4, zp)

    plt.figure(3)

    plt.xlim([-4, 4])
    plt.ylim([zrange[0], zrange[1]])
    plt.xlabel('X [Å]')
    plt.ylabel(zlabel)

    plt.plot(x, f1(x, 0), label=label1)
    plt.plot(x, f2(x, 0), label=label2)
    plt.legend()

    if pdf is not None:
        pdf.savefig()

    if show_plot:
        plt.show()
    else:
        plt.close()


def biplot(data1, data2, label1, label2, y_range, z_range, pdf=None,
           zlabel='Energy [eV]', zrange=(-1.5, 1.5), title=None, show_plot=True,
           direction=0):

    plt.figure(3)

    if direction == 0:
        data1 = np.array(data1).reshape([len(y_range), len(z_range)])[len(z_range)//2]
        data2 = np.array(data2).reshape([len(y_range), len(z_range)])[len(z_range)//2]
        plt.xlabel('X [Å]')

    if direction == 1:
        data1 = np.array(data1).reshape([len(y_range), len(z_range)])[:,len(y_range)//2]
        data2 = np.array(data2).reshape([len(y_range), len(z_range)])[:,len(y_range)//2]
        plt.xlabel('Y [Å]')

    plt.title(title)
    plt.xlim([-4, 4])
    if zrange is not None:
        plt.ylim([zrange[0], zrange[1]])
    plt.ylabel(zlabel)

    plt.plot(z_range, data1, label=label1)
    plt.plot(z_range, data2, label=label2)
    plt.legend()

    if pdf is not None:
        pdf.savefig()

    if show_plot:
        plt.show()
    else:
        plt.close()


def multibiplot(data, labels, y_range, z_range, pdf=None,
           zlabel='Energy [eV]', zrange=(-1.5, 1.5), title=None, show_plot=True,
           direction=0):

    plt.figure(3)

    if direction == 0:
        for i, dat in enumerate(data):
            data[i] = np.array(dat).reshape([len(y_range), len(z_range)])[len(z_range)//2]
            #data[i] = np.array(dat).reshape([len(y_range), len(z_range)])[len(z_range)//2]
            plt.xlabel('X [Å]')

    if direction == 1:
        for i, dat in enumerate(data):

            data[i] = np.array(dat).reshape([len(y_range), len(z_range)])[:,len(y_range)//2]
            # data2 = np.array(data2).reshape([len(y_range), len(z_range)])[:,len(y_range)//2]
            plt.xlabel('Y [Å]')

    plt.title(title)
    plt.xlim([-4, 4])
    if zrange is not None:
        plt.ylim([zrange[0], zrange[1]])
    plt.ylabel(zlabel)
    for dat, l in zip(data, labels):
        plt.plot(z_range, dat, label=l)
        # plt.plot(z_range, data2, label=label2)
    plt.legend()

    if pdf is not None:
        pdf.savefig()

    if show_plot:
        plt.show()
    else:
        plt.close()


def multibiplot_2axis(data, labels, data2, labels2, y_range, z_range, pdf=None,
           zlabel='Energy [eV]', zlabel2=' ', zrange=(-1.5, 1.5), zrange2=(-1.5, 1.5),
           title=None, show_plot=True, direction=0):

    #plt.figure(3)
    f, ax1 = plt.subplots()
    ax2 = ax1.twinx()

    if direction == 0:
        ax1.set_xlabel('X [Å]')
        for i, dat in enumerate(data):
            data[i] = np.array(dat).reshape([len(y_range), len(z_range)])[len(z_range)//2]
        for i, dat in enumerate(data2):
            data2[i] = np.array(dat).reshape([len(y_range), len(z_range)])[len(z_range)//2]

    if direction == 1:
        ax1.set_xlabel('Y [Å]')
        for i, dat in enumerate(data):
            data[i] = np.array(dat).reshape([len(y_range), len(z_range)])[:,len(y_range)//2]
            # data2 = np.array(data2).reshape([len(y_range), len(z_range)])[:,len(y_range)//2]
        for i, dat in enumerate(data2):
            data2[i] = np.array(dat).reshape([len(y_range), len(z_range)])[:,len(y_range)//2]

    plt.title(title)
    ax1.set_xlim([-4, 4])
    if zrange is not None:
        ax1.set_ylim([zrange[0], zrange[1]])
        ax1.set_ylabel(zlabel)
    if zrange2 is not None:
        ax2.set_ylim([zrange[0], zrange[1]])
        ax2.set_ylabel(zlabel2)

    lns1 = []
    for dat, l in zip(data, labels):
        lns1.append(ax1.plot(z_range, dat, label=l))
        # plt.plot(z_range, data2, label=label2)

    lns2 = []
    for dat, l in zip(data2, labels2):
        lns2.append(ax2.plot(z_range, dat, label=l))

    lns = [j for i in lns1 for j in i] + [j for i in lns2 for j in i]
    labs = [l.get_label() for l in lns]
    plt.legend(lns, labs, loc=0)

    if pdf is not None:
        pdf.savefig()

    if show_plot:
        plt.show()
    else:
        plt.close()



def v2_mayavi(data1, data2, label1, label2, y_range, z_range,):
    transparency = False

    from mayavi import mlab
    fig = mlab.figure()

    x = np.arange(-2, 2, 0.1)
    y = np.arange(-2, 2, 0.1)
    mx, my = np.meshgrid(x, y, indexing='ij')
    mz1 = np.abs(mx) + np.abs(my)
    mz2 = mx ** 2 + my ** 2

    print(mx.shape)
    print(my.shape)
    print(mz1.shape)


    my, mx = np.meshgrid(z_range, y_range)
    mz1 = np.array(data1).reshape([len(y_range), len(z_range)])
    mz2 = np.array(data2).reshape([len(y_range), len(z_range)])

    print(my.shape)
    print(mx.shape)
    print(mz1.shape)

    ax_ranges = [-4, 4, -4, 4, -0.4, 0.4]
    ax_scale = [1.0, 1.0, 10]
    ax_extent = ax_ranges * np.repeat(ax_scale, 2)

    surf3 = mlab.surf(mx, my, mz1, colormap='Blues')
    surf4 = mlab.surf(mx, my, mz2, colormap='Oranges')

    surf3.actor.actor.scale = ax_scale
    surf4.actor.actor.scale = ax_scale
    mlab.view(60, 74, 17, [-2.5, -4.6, -0.3])
    mlab.outline(surf3, color=(.7, .7, .7), extent=ax_extent)
    mlab.axes(surf3, color=(.7, .7, .7), extent=ax_extent,
              ranges=ax_ranges,
              xlabel='x', ylabel='y', zlabel='z')

    if transparency:
        surf3.actor.property.opacity = 0.5
        surf4.actor.property.opacity = 0.5
        fig.scene.renderer.use_depth_peeling = 1

    mlab.show()


#############################
folder = args.output_folder
#############################

with open(args.filename, 'rb') as input:
    calculation_data = pickle.load(input)
    print('Loaded data from: {}'.format(args.filename))

do_full = args.full
interpolate = args.interpolate

for slide_y in calculation_data['range_y']:
    for slide_z in calculation_data['range_z']:
        print(slide_y, slide_z)
        if '{}_{}'.format(slide_y, slide_z) in calculation_data:
            print(calculation_data['{}_{}'.format(slide_y, slide_z)])
            data_i = calculation_data['{}_{}'.format(slide_y, slide_z)]
            print(data_i['states_info'])
            if do_full:

                calculation_data.update({'{}_{}'.format(-slide_y, slide_z): data_i,
                                         '{}_{}'.format(slide_y, -slide_z): data_i,
                                         '{}_{}'.format(-slide_y, -slide_z): data_i})


if do_full:
    y_range = np.unique((-np.array(calculation_data['range_y'])).tolist() + calculation_data['range_y'])
    z_range = np.unique((-np.array(calculation_data['range_z'])).tolist() + calculation_data['range_z'])
else:
    y_range = calculation_data['range_y']
    z_range = calculation_data['range_z']

points = []
total_data = []
full_range = []
i = 0
for slide_y in y_range:
    for slide_z in z_range:
        print('{}_{}'.format(slide_y, slide_z))
        if '{}_{}'.format(slide_y, slide_z) in calculation_data:
            data = calculation_data['{}_{}'.format(slide_y, slide_z)]
            total_data.append(data)
            full_range.append(i)
            points.append([slide_y, slide_z])
            i += 1

states_orders = [get_order_states_list(data['states_info']) for data in total_data]

########################  W_DC  ######################

data_1 = []
data_2 = []
data_3 = []
data_4 = []

for diab, coeff in [[data['diabatic_energies'], data['coefficients']] for data in total_data]:

    factor = coeff['S_01'][0] * coeff['S_10'][0]
    data_1.append(diab['V_DC'] * np.sign(factor))

    factor = coeff['S_01'][1] * coeff['S_10'][1]
    data_2.append(diab['V_DC'] * np.sign(factor))

    factor = coeff['S_01'][2] * coeff['S_10'][2]
    data_3.append(diab['V_DC'] * np.sign(factor))

    factor = coeff['S_01'][3] * coeff['S_10'][3]
    data_4.append(diab['V_DC'] * np.sign(factor))

data_1 = [data['diabatic_contributions']['W_DC'][0] for data in total_data]
data_2 = [data['diabatic_contributions']['W_DC'][1] for data in total_data]
data_3 = [data['diabatic_contributions']['W_DC'][2] for data in total_data]
data_4 = [data['diabatic_contributions']['W_DC'][3] for data in total_data]

data_1, data_2, data_3, data_4 = correct_order_list([data_1, data_2, data_3, data_4], states_orders)

if interpolate:
    data_1 = interpolate_data(points, data_1, y_range, z_range)
    data_2 = interpolate_data(points, data_2, y_range, z_range)

wdc1 = np.array(data_1)
wdc2 = np.array(data_2)


#v2_mayavi(data_1, data_2, '$W^{(1)}_{DC}$', '$W^{(2)}_{DC}$',  y_range, z_range)
#exit()

with PdfPages(folder + 'W_DC.pdf') as pdf:
    triplot(data_1, data_2, '$W^{(1)}_{DC}$', '$W^{(2)}_{DC}$', y_range, z_range, pdf=pdf, wireframe=True,
            show_plot=args.show_plots, zlevels=np.arange(-0.4, 0.4 + 0.05, 0.05))
            #show_plot = args.show_plots, zlevels = np.arange(-0.4, 0.4 + 0.05, 0.05))

    biplot(data_1, data_2, '$W^{(1)}_{DC}$', '$W^{(2)}_{DC}$', y_range, z_range, show_plot=args.show_plots,
           direction=0, pdf=pdf)
    biplot(data_1, data_2, '$W^{(1)}_{DC}$', '$W^{(2)}_{DC}$', y_range, z_range, show_plot=args.show_plots,
           direction=1, pdf=pdf)



########################  W_CT  ######################

data_1 = [data['diabatic_contributions']['W_CT'][0] for data in total_data]
data_2 = [data['diabatic_contributions']['W_CT'][1] for data in total_data]
data_3 = [data['diabatic_contributions']['W_CT'][2] for data in total_data]
data_4 = [data['diabatic_contributions']['W_CT'][3] for data in total_data]

data_1, data_2, data_3, data_4 = correct_order_list([data_1, data_2, data_3, data_4], states_orders)

if interpolate:
    data_1 = interpolate_data(points, data_1, y_range, z_range)
    data_2 = interpolate_data(points, data_2, y_range, z_range)

wct1 = np.array(data_1)
wct2 = np.array(data_2)

with PdfPages(folder + 'W_CT.pdf') as pdf:
    triplot(data_1, data_2, '$W^{(1)}_{CT}$', '$W^{(2)}_{CT}$', y_range, z_range, pdf=pdf, wireframe=True, show_plot=args.show_plots)
    biplot(data_1, data_2, '$W^{(1)}_{CT}$', '$W^{(2)}_{CT}$', y_range, z_range, show_plot=args.show_plots, pdf=pdf, direction=0)
    biplot(data_1, data_2, '$W^{(1)}_{CT}$', '$W^{(2)}_{CT}$', y_range, z_range, show_plot=args.show_plots, pdf=pdf, direction=1)



########################  W_e  ######################


data_1 = [data['diabatic_contributions']['W_e'][0] for data in total_data]
data_2 = [data['diabatic_contributions']['W_e'][1] for data in total_data]
data_3 = [data['diabatic_contributions']['W_e'][2] for data in total_data]
data_4 = [data['diabatic_contributions']['W_e'][3] for data in total_data]

data_1, data_2, data_3, data_4 = correct_order_list([data_1, data_2, data_3, data_4], states_orders)

if interpolate:
    data_1 = interpolate_data(points, data_1, y_range, z_range)
    data_2 = interpolate_data(points, data_2, y_range, z_range)

we1 = np.array(data_1)
we2 = np.array(data_2)

with PdfPages(folder + 'W_e.pdf') as pdf:
    triplot(data_1, data_2, 'W_e_1', 'W_e_2', y_range, z_range, pdf=pdf, wireframe=True, show_plot=args.show_plots)
    biplot(data_1, data_2, 'W_e_1', 'W_e_2', y_range, z_range, show_plot=args.show_plots, pdf=pdf, direction=0)
    biplot(data_1, data_2, 'W_e_1', 'W_e_2', y_range, z_range, show_plot=args.show_plots, pdf=pdf, direction=1)


#######################  W_h  ######################


data_1 = [data['diabatic_contributions']['W_h'][0] for data in total_data]
data_2 = [data['diabatic_contributions']['W_h'][1] for data in total_data]
data_3 = [data['diabatic_contributions']['W_h'][2] for data in total_data]
data_4 = [data['diabatic_contributions']['W_h'][3] for data in total_data]

data_1, data_2, data_3, data_4 = correct_order_list([data_1, data_2, data_3, data_4], states_orders)

if interpolate:
    data_1 = interpolate_data(points, data_1, y_range, z_range)
    data_2 = interpolate_data(points, data_2, y_range, z_range)

wh1 = np.array(data_1)
wh2 = np.array(data_2)

with PdfPages(folder + 'W_h.pdf') as pdf:
    triplot(data_1, data_2, 'W_h_1', 'W_h_2', y_range, z_range, pdf=pdf, wireframe=True, show_plot=args.show_plots)
    biplot(data_1, data_2, 'W_h_1', 'W_h_2', y_range, z_range, show_plot=args.show_plots, pdf=pdf, direction=0)
    biplot(data_1, data_2, 'W_h_1', 'W_h_2', y_range, z_range, show_plot=args.show_plots, pdf=pdf, direction=1)

#######################  diabatic_energies  ######################

data_1 = [np.average(data['diabatic_energies']['E_LE']) for data in total_data]
data_2 = [np.average(data['diabatic_energies']['E_CT']) for data in total_data]

if interpolate:
    data_1 = interpolate_data(points, data_1, y_range, z_range)
    data_2 = interpolate_data(points, data_2, y_range, z_range)

e_le = np.array(data_1)
e_ct = np.array(data_2)

with PdfPages(folder + 'diabatic_energies.pdf') as pdf:
    triplot(data_1, data_2, '$E_{LE}$', '$E_{CT}$', y_range, z_range, pdf=pdf, wireframe=True, show_plot=args.show_plots, zlevels=None)
    biplot(data_1, data_2, '$E_{LE}$', '$E_{CT}$', y_range, z_range, show_plot=args.show_plots, zrange=None, pdf=pdf, direction=0)
    biplot(data_1, data_2, '$E_{LE}$', '$E_{CT}$', y_range, z_range, show_plot=args.show_plots, zrange=None, pdf=pdf, direction=1)


#######################  lambda  ######################

data_1 = [data['lambda'][0] for data in total_data]
data_2 = [data['lambda'][1] for data in total_data]
data_3 = [data['lambda'][2] for data in total_data]
data_4 = [data['lambda'][3] for data in total_data]

data_1, data_2, data_3, data_4 = correct_order_list([data_1, data_2, data_3, data_4], states_orders)

if interpolate:
    data_1 = interpolate_data(points, data_1, y_range, z_range)
    data_2 = interpolate_data(points, data_2, y_range, z_range)

l1 = np.array(data_1)
l2 = np.array(data_2)

with PdfPages(folder + 'lambda.pdf') as pdf:
    triplot(data_1, data_2, 'lambda 1', 'lambda 2', y_range, z_range, pdf=pdf, wireframe=True, show_plot=args.show_plots)
    biplot(data_1, data_2, 'lambda 1', 'lambda 2', y_range, z_range, show_plot=args.show_plots, pdf=pdf, direction=0)
    biplot(data_1, data_2, 'lambda 1', 'lambda 2', y_range, z_range, show_plot=args.show_plots, pdf=pdf, direction=1)

with PdfPages(folder + 'lambda2.pdf') as pdf:
    triplot(np.square(data_1), np.square(data_2), 'lambda^2 1', 'lambda^2 2', y_range, z_range, pdf=pdf, wireframe=True,
            show_plot=args.show_plots)
    biplot(np.square(data_1), np.square(data_2), 'lambda^2 1', 'lambda^2 2', y_range, z_range,
           show_plot=args.show_plots, pdf=pdf, direction=0)
    biplot(np.square(data_1), np.square(data_2), 'lambda^2 1', 'lambda^2 2', y_range, z_range,
           show_plot=args.show_plots, pdf=pdf, direction=1)



##########################  OMEGA h  #######################

wh1_ = np.array([data['diabatic_contributions']['W_h'][0] for data in total_data])
wh2_ = np.array([data['diabatic_contributions']['W_h'][1] for data in total_data])

l1_ = np.array([data['lambda'][0] for data in total_data])
l2_ = np.array([data['lambda'][1] for data in total_data])

data_1 = 2 * l1_ * np.sqrt(1 - l1_**2) * wh1_
data_2 = 2 * l2_ * np.sqrt(1 - l2_**2) * wh2_

data_1, data_2, data_3, data_4 = correct_order_list([data_1, data_2, data_3, data_4], states_orders)

if interpolate:
    data_1 = interpolate_data(points, data_1, y_range, z_range)
    data_2 = interpolate_data(points, data_2, y_range, z_range)

with PdfPages(folder + 'omega_h.pdf') as pdf:
    triplot(data_1, data_2, 'omega_h 1', 'omaga_h 2', y_range, z_range, wireframe=True, pdf=pdf, show_plot=args.show_plots)
    biplot(data_1, data_2, 'state 1', 'state 2', y_range, z_range, pdf=pdf, show_plot=args.show_plots, title='omega_h', direction=0)
    biplot(data_1, data_2, 'state 1', 'state 2', y_range, z_range, pdf=pdf, show_plot=args.show_plots, title='omega_h', direction=1)

oh1 = np.array(data_1)
oh2 = np.array(data_2)


##########################  OMEGA e  #######################

we1_ = np.array([data['diabatic_contributions']['W_e'][0] for data in total_data])
we2_ = np.array([data['diabatic_contributions']['W_e'][1] for data in total_data])

l1_ = np.array([data['lambda'][0] for data in total_data])
l2_ = np.array([data['lambda'][1] for data in total_data])

data_1 = 2 * l1_ * np.sqrt(1 - l1_**2) * we1_
data_2 = 2 * l2_ * np.sqrt(1 - l2_**2) * we2_

data_1, data_2, data_3, data_4 = correct_order_list([data_1, data_2, data_3, data_4], states_orders)

if interpolate:
    data_1 = interpolate_data(points, data_1, y_range, z_range)
    data_2 = interpolate_data(points, data_2, y_range, z_range)


with PdfPages(folder + 'omega_e.pdf') as pdf:
    triplot(data_1, data_2, 'omega_e 1', 'omaga_e 2', y_range, z_range, wireframe=True, pdf=pdf, show_plot=args.show_plots)
    biplot(data_1, data_2, 'state 1', 'state 2', y_range, z_range, pdf=pdf, show_plot=args.show_plots, title='omega_e', direction=0)
    biplot(data_1, data_2, 'state 1', 'state 2', y_range, z_range, pdf=pdf, show_plot=args.show_plots, title='omega_e', direction=1)

oe1 = np.array(data_1)
oe2 = np.array(data_2)

##########################  SUPEREXCHANGE(E1) #######################

#data_1 = 2 * l1 * np.sqrt(1 - l1**2) * (np.array(we1) + np.array(wh1))
#data_2 = 2 * l2 * np.sqrt(1 - l2**2) * (np.array(we2) + np.array(wh2))

data_1 = oh1 + oe1
data_2 = oh2 + oe2

with PdfPages(folder + 'E1(superexchange).pdf') as pdf:
    triplot(data_1, data_2, '$\Omega^{(1)}$', '$\Omega^{(2)}$', y_range, z_range, wireframe=True, pdf=pdf,
            show_plot=args.show_plots, zlevels=np.arange(-1.5, 1.5 + 0.125, 0.125))
    triplot(data_1, data_2, '$\Omega^{(1)}$', '$\Omega^{(2)}$', y_range, z_range, wireframe=True, pdf=pdf,
            show_plot=args.show_plots, zlevels=np.arange(-0.2, 0.2 + 0.025, 0.025))
    biplot(data_1, data_2, '$\Omega^{(1)}$', '$\Omega^{(2)}$', y_range, z_range, pdf=pdf,
           show_plot=args.show_plots, title='$\Omega$', direction=0, zrange=[-1.5, 1.5])
    biplot(data_1, data_2, '$\Omega^{(1)}$', '$\Omega^{(2)}$', y_range, z_range, pdf=pdf,
           show_plot=args.show_plots, title='$\Omega$', direction=1, zrange=[-1.5, 1.5])

e_11 = np.array(data_1)
e_12 = np.array(data_2)


#######################  second order term (E2)  ######################

data_1 = l1**2 * (e_ct - e_le + wct1 - wdc1)
data_2 = l2**2 * (e_ct - e_le + wct2 - wdc2)


with PdfPages(folder + 'E2.pdf') as pdf:
    triplot(data_1, data_2, '$E(\lambda^2_1)$', '$E(\lambda^2_2)$', y_range, z_range,
            wireframe=True, pdf=pdf, show_plot=args.show_plots, zlevels=np.arange(-0.4, 0.4 + 0.05, 0.05))
    triplot(data_1, data_2, '$E(\lambda^2_1)$', '$E(\lambda^2_2)$', y_range, z_range,
    wireframe=True, pdf=pdf, show_plot=args.show_plots, zlevels=np.arange(-0.1, 0.1 + 0.02, 0.02))
    multibiplot([data_1, data_2, np.square(l1), np.square(l2)],
                ['$E(\lambda^2_1)$', '$E(\lambda^2_2)$', '$\lambda^2_1$', '$\lambda^2_2$'], y_range, z_range,
                show_plot=args.show_plots, pdf=pdf, direction=0)
    multibiplot([data_1, data_2, np.square(l1), np.square(l2)],
                ['$E(\lambda^2_1)$', '$E(\lambda^2_2)$', '$\lambda^2_1$', '$\lambda^2_2$'], y_range, z_range,
                show_plot=args.show_plots, pdf=pdf, direction=1)


    #biplot(data_1, data_2, '$E(\lambda^2_1)$', '$E(\lambda^2_2)$', y_range, z_range, show_plot=args.show_plots, pdf=pdf, direction=0)
    #biplot(data_1, data_2, '$E(\lambda^2_1)$', '$E(\lambda^2_2)$', y_range, z_range, show_plot=args.show_plots, pdf=pdf, direction=1)

e_21 = np.array(data_1)
e_22 = np.array(data_2)
with PdfPages(folder + 'figure5.pdf') as pdf:

    multibiplot_2axis([wdc1, e_11, e_21], ['$W^{(1)}_{DC}$', '$\Omega^{(1)}$','$E(\lambda^2_1)$'],
                      [np.square(l1)], [ '$\lambda^2_1$'], y_range, z_range, pdf=pdf, zlabel2='$\lambda^2$',
                      zlabel='Energy [eV]', zrange=(-1.5, 1.5), zrange2=(-1.5, 1.5), title='State 1',
                      show_plot=args.show_plots, direction=0)

    multibiplot_2axis([wdc1, e_11, e_21], ['$W^{(1)}_{DC}$', '$\Omega^{(1)}$','$E(\lambda^2_1)$'],
                      [np.square(l1)], [ '$\lambda^2_1$'], y_range, z_range, pdf=pdf, zlabel2='$\lambda^2$',
                      zlabel='Energy [eV]', zrange=(-1.5, 1.5), zrange2=(-1.5, 1.5), title='State 1',
                      show_plot=args.show_plots, direction=1)

    multibiplot_2axis([wdc2, e_12, e_22], ['$W^{(2)}_{DC}$', '$\Omega^{(2)}$','$E(\lambda^2_2)$'],
                      [np.square(l2)], [ '$\lambda^2_2$'], y_range, z_range, pdf=pdf, zlabel2='$\lambda^2$',
                      zlabel='Energy [eV]', zrange=(-1.5, 1.5), zrange2=(-1.5, 1.5), title='State 1',
                      show_plot=args.show_plots, direction=0)

    multibiplot_2axis([wdc2, e_12, e_22], ['$W^{(2)}_{DC}$', '$\Omega^{(2)}$','$E(\lambda^2_2)$'],
                      [np.square(l2)], [ '$\lambda^2_2$'], y_range, z_range, pdf=pdf, zlabel2='$\lambda^2$',
                      zlabel='Energy [eV]', zrange=(-1.5, 1.5), zrange2=(-1.5, 1.5), title='State 1',
                      show_plot=args.show_plots, direction=1)

#######################  adiabatic_energies calculated ######################

e1 = e_le + wdc1 + e_11 + e_21
e2 = e_le + wdc2 + e_12 + e_22


with PdfPages(folder + 'adiabatic_energies.pdf') as pdf:
    triplot(e1, e2, 'e1', 'e2', y_range, z_range, pdf=pdf, wireframe=True, show_plot=args.show_plots, zlevels=None)
    biplot(e1, e2, 'e1', 'e2', y_range, z_range, show_plot=args.show_plots, zrange=None, pdf=pdf, direction=0)
    biplot(e1, e2, 'e1', 'e2', y_range, z_range, show_plot=args.show_plots, zrange=None, pdf=pdf, direction=1)


#######################  adiabatic_energies extracted ######################

data_1 = [data['adiabatic_energies']['E_1'] for data in total_data]
data_2 = [data['adiabatic_energies']['E_2'] for data in total_data]
data_3 = [data['adiabatic_energies']['E_3'] for data in total_data]
data_4 = [data['adiabatic_energies']['E_4'] for data in total_data]

data_1, data_2, data_3, data_4 = correct_order_list([data_1, data_2, data_3, data_4], states_orders)

if interpolate:
    data_1 = interpolate_data(points, data_1, y_range, z_range)
    data_2 = interpolate_data(points, data_2, y_range, z_range)

with PdfPages(folder + 'adiabatic_energies.pdf') as pdf:
    triplot(data_1, data_2, '$E_1$', '$E_2$', y_range, z_range, pdf=pdf, wireframe=True, show_plot=args.show_plots, zlevels=None)
    biplot(data_1, data_2, '$E_1$', '$E_2$', y_range, z_range, show_plot=args.show_plots, zrange=None, pdf=pdf, direction=0)
    biplot(data_1, data_2, '$E_1$', '$E_2$', y_range, z_range, show_plot=args.show_plots, zrange=None, pdf=pdf, direction=1)


#######################  difference test ######################

with PdfPages(folder + 'adiabatic_energies_diff.pdf') as pdf:
    triplot(data_1-e1, data_2-e2, 'diff e1', 'diff e2', y_range, z_range, pdf=pdf, wireframe=True, show_plot=args.show_plots, zlevels=None)
    biplot(data_1-e1, data_2-e2, 'diff e1', 'diff e2', y_range, z_range, show_plot=args.show_plots,
           zrange=None,  title='Adiabatic energy differences (original - calculated)')




__author__ = 'Abel Carreras'
AU_TO_EV = 27.21138

from pyqchem.structure import Structure
from pyqchem.errors import ParserError
from pyqchem.parsers.common import search_bars, standardize_vector
from pyqchem.parsers.common import read_basic_info, get_cis_occupations_list, read_symmetry_info
import numpy as np
import re


def _list_to_complex(list):
    real = list[0].replace('(', '').replace(')', '')
    if real[:2] == '--':
        real = real[2:]

    imag = list[2].split('i')[0].replace('(', '')
    if imag[:2] == '--':
        imag = imag[2:]

    opera = float(list[1] + '1')
    return float(real) + opera * float(imag) * 1.j


def basic_cis(output):
    """
    Parser for CIS/TD-DFT calculations

    :param output:
    :return:
    """
    data_dict = {}

    # Molecule
    n = output.find('$molecule')
    n2 = output[n:].find('$end')

    molecule_region = output[n:n+n2-1].replace('\t', ' ').split('\n')[1:]
    charge, multiplicity = [int(num) for num in molecule_region[0].split()]
    coordinates = [[float(l) for l in line.split()[1:4]] for line in molecule_region[1:]]
    symbols = [line.split()[0].capitalize() for line in molecule_region[1:]]
    n_atoms = len(symbols)

    # structure
    structure_input = Structure(coordinates=coordinates,
                                symbols=symbols,
                                charge=charge,
                                multiplicity=multiplicity)

    enum = output.find('Standard Nuclear Orientation')
    section_structure = output[enum:enum + 200*structure_input.get_number_of_atoms()].split('\n')
    section_structure = section_structure[3:structure_input.get_number_of_atoms()+3]
    coordinates = [[float(num) for num in s.split()[2:]] for s in section_structure]

    data_dict['structure'] = Structure(coordinates=coordinates,
                                       symbols=symbols,
                                       charge=charge,
                                       multiplicity=multiplicity)

    # scf_energy
    enum = output.find('Total energy in the final basis set')
    try:
        data_dict['scf_energy'] = float(output[enum:enum+100].split()[8])
    except IndexError:
        pass

    enum = output.find('Molecular Point Group')
    if enum > 0:
        symmetry_data = read_symmetry_info(output[enum:enum + 1000])

    enum = output.find('Nuclear Repulsion Energy')
    basic_data = read_basic_info(output[enum:enum + 5000])

    # CIS excited states
    # enum = output.find('CIS Excitation Energies')
    try:
        enum = list(re.finditer('CIS Excitation Energies', output))[-1].end()
    except IndexError:
        enum = list(re.finditer('TDDFT/TDA Excitation Energies', output))[-1].end()

    excited_states = []
    if enum > 0:
        bars = search_bars(output, from_position=enum)

        output_cis = output[bars[0]:bars[1]]

        for m in re.finditer('Excited state ', output_cis):
            state_cis_section = output_cis[m.end():]
            state_cis_lines = state_cis_section.split('\n')

            exc_energy = float(state_cis_lines[0].split()[5])
            exc_energy_units = state_cis_lines[0].split()[3][1:-1]
            tot_energy = float(state_cis_lines[1].split()[5])

            try:
                tot_energy_units = state_cis_lines[1].split()[6]
                mul = state_cis_lines[2].split()[-1]

                trans_mom = [float(mom) for mom in [state_cis_lines[3].split()[2],
                                                    state_cis_lines[3].split()[4],
                                                    state_cis_lines[3].split()[6]]]
                strength = float(state_cis_lines[4].split()[2])
            except ValueError:
                # old version of qchem (< 5.01)
                state_cis_words = output_cis[m.end():].split()
                tot_energy_units = 'au'
                mul = state_cis_words[13]
                trans_mom = [float(mom) for mom in [state_cis_words[16],
                                                    state_cis_words[18],
                                                    state_cis_words[20]]]
                strength = float(state_cis_words[24])

            transitions = []
            for line in state_cis_lines[5:]:
                if line.find('-->') > 0:
                    origin = int(line.split('>')[0].split('(')[1].split(')')[0])
                    target = int(line.split('>')[1].split('(')[1].split(')')[0])
                    amplitude = float(line.split('=')[1].split()[0])

                    alpha_transitions = []
                    beta_transitions = []
                    try:
                        spin = line[21:].split()[3]
                        if spin == 'alpha':
                            alpha_transitions.append({'origin': origin, 'target': target + basic_data['n_alpha']})
                        elif spin == 'beta':
                            beta_transitions.append({'origin': origin, 'target': target + basic_data['n_beta']})
                        else:
                            raise ParserError('basic_cis', 'Error reading configurations')

                        transitions.append({'origin': origin,
                                            'target': target,
                                            'amplitude': amplitude,
                                            'occupations': get_cis_occupations_list(basic_data['n_basis_functions'],
                                                                                    basic_data['n_alpha'],
                                                                                    basic_data['n_beta'],
                                                                                    alpha_transitions=alpha_transitions,
                                                                                    beta_transitions=beta_transitions)})

                    except (IndexError, ParserError):
                        # This supposes single electron transition
                        alpha_transitions.append({'origin': origin, 'target': target + basic_data['n_alpha']})

                        transitions.append({'origin': origin,
                                            'target': target,
                                            'amplitude': amplitude/np.sqrt(2),
                                            'occupations': get_cis_occupations_list(basic_data['n_basis_functions'],
                                                                                    basic_data['n_alpha'],
                                                                                    basic_data['n_beta'],
                                                                                    alpha_transitions=alpha_transitions,
                                                                                    beta_transitions=beta_transitions)})

                        transitions.append({'origin': origin,
                                            'target': target,
                                            'amplitude': amplitude/np.sqrt(2) if mul == 'Singlet' else -amplitude/np.sqrt(2),
                                            'occupations': get_cis_occupations_list(basic_data['n_basis_functions'],
                                                                                    basic_data['n_alpha'],
                                                                                    basic_data['n_beta'],
                                                                                    alpha_transitions=beta_transitions,
                                                                                    beta_transitions=alpha_transitions)})

                if len(line) < 5:
                    break

            excited_states.append({'total_energy': tot_energy,
                                   'total_energy_units': tot_energy_units,
                                   'excitation_energy': exc_energy,
                                   'excitation_energy_units': exc_energy_units,
                                   'multiplicity': mul,
                                   'transition_moment': standardize_vector(trans_mom),
                                   'strength': strength,
                                   'configurations': transitions})

    data_dict['excited_states'] = excited_states

    # Spin-Orbit coupling
    initial = output.find('*********SPIN-ORBIT COUPLING JOB BEGINS HERE*********')
    final = output.find('*********SOC CODE ENDS HERE*********')

    data_interstate = {}
    if initial > 0:
        soc_section = output[initial:final]

        def label_states(excited_states):
            labels = []
            ns = 1
            nt = 1
            for state in excited_states:
                if state['multiplicity'].lower() == 'singlet':
                    labels.append('S{}'.format(ns))
                    ns += 1
                elif state['multiplicity'].lower() == 'triplet':
                    labels.append('T{}'.format(nt))
                    nt += 1
                else:
                    try:
                        m = float(state['multiplicity'])
                        if abs(m - 1) < 0.1:
                            labels.append('S{}'.format(ns))
                            ns += 1
                        if abs(m - 3) < 0.1:
                            labels.append('T{}'.format(nt))
                            nt += 1
                        state['multiplicity'] = m

                    except ValueError:
                        raise ParserError('basic_cis', 'State multiplicity error')

            return labels, nt-1, ns-1

        labels, n_triplet, n_singlet = label_states(excited_states)

        for i, label in enumerate(labels):
            data_interstate[(i+1, 0)] = {'1e_soc_mat': [0j, 0j, 0j], 'soc_units': 'cm-1'}
            data_interstate[(0, i+1)] = {'1e_soc_mat': [0j, 0j, 0j], 'soc_units': 'cm-1'}
            for j, label2 in enumerate(labels):
                if (label[0] == 'S' or label2[0] == 'S') and (label[0] != label2[0]):
                    data_interstate[(i+1, j+1)] = {'1e_soc_mat': [[0j, 0j, 0j]], 'soc_units': 'cm-1'}
                elif label[0] == 'T' and label2[0] == 'T':
                    data_interstate[(i+1, j+1)] = {'1e_soc_mat': [[0j, 0j, 0j], [0j, 0j, 0j], [0j, 0j, 0j]], 'soc_units': 'cm-1'}
                elif label[0] == 'S' and label2[0] == 'S':
                    data_interstate[(i+1, j+1)] = {'1e_soc_mat': [[0j]], 'soc_units': 'cm-1'}
                else:
                    raise ParserError('basic_cis', 'State multiplicity error')

        for i, label in enumerate(labels):
            for k2, ms2 in enumerate([-1, 0, 1]):
                for j, label2 in enumerate(labels):
                    if label[0] == 'T':
                        for k, ms in enumerate([-1, 0, 1]):
                            enum = soc_section.find('SOC between the {} (ms={}) state and excited triplet states (ms={})'.format(label, ms2, ms))
                            for line in soc_section[enum:enum+50*(n_triplet+1)].split('\n'):
                                if len(line.split()) == 0:
                                    break
                                if line.split()[0] == '{}(ms={})'.format(label2, ms):
                                    data_interstate[(i+1, j+1)]['1e_soc_mat'][k2][k] = _list_to_complex(line.split()[1:4])
                                    data_interstate[(i+1, j+1)]['1e_soc_mat'][k][k2] = _list_to_complex(line.split()[1:4])
                                    data_interstate[(j+1, i+1)]['1e_soc_mat'][k2][k] = _list_to_complex(line.split()[1:4])
                                    data_interstate[(j+1, i+1)]['1e_soc_mat'][k][k2] = _list_to_complex(line.split()[1:4])
                                    break

                    elif label[0] == 'S':
                        for k, ms in enumerate([-1, 0, 1]):
                            enum = soc_section.find('SOC between the {} state and excited triplet states (ms={})'.format(label, ms))
                            for line in soc_section[enum:enum+50*(n_triplet+1)].split('\n'):
                                if len(line.split()) == 0:
                                    break
                                if line.split()[0] == '{}(ms={})'.format(label2, ms):
                                    data_interstate[(i+1, j+1)]['1e_soc_mat'][0][k] = _list_to_complex(line.split()[1:4])
                                    data_interstate[(j+1, i+1)]['1e_soc_mat'][0][k] = _list_to_complex(line.split()[1:4])
                                    break
                    else:
                        raise ParserError('basic_cis', 'SOC reading error')

                enum = soc_section.find('SOC between the singlet ground state and excited triplet states (ms={})'.format(ms2))
                for line in soc_section[enum:enum+50*(n_triplet+1)].split('\n'):
                    if len(line.split()) == 0:
                        break
                    if line.split()[0] == '{}(ms={})'.format(label, ms2):
                        data_interstate[(i+1, 0)]['1e_soc_mat'][k2] = _list_to_complex(line.split()[1:4])
                        data_interstate[(0, i+1)]['1e_soc_mat'][k2] = _list_to_complex(line.split()[1:4])
                        break

        data_dict['interstate_properties'] = data_interstate

    # diabatization
    initial = output.find('Localization Code for CIS excited states')
    if initial > 0:

        bars = search_bars(output, from_position=initial)
        diabat_section = output[initial: bars[0]]

        def read_diabatization_matrix(label):
            matrix = []
            for m in re.finditer(label, diabat_section):
                line = diabat_section[m.end(): m.end() + 50].split('\n')[0]
                matrix.append(float(line.split('=')[1]))

            diabat_dim = int(np.sqrt(len(matrix)))
            return np.array(matrix).reshape(diabat_dim, diabat_dim).T

        r_matrix = read_diabatization_matrix('showmatrix adiabatic R-Matrix')
        rot_matrix = read_diabatization_matrix('showmatrix final adiabatic -> diabatic RotMatrix')
        adiabatic_matrix = read_diabatization_matrix('showmatrix adiabatH') * AU_TO_EV
        diabatic_matrix = read_diabatization_matrix('showmatrix diabatH') * AU_TO_EV

        diabat_data = {'rot_matrix': rot_matrix,
                       'adiabatic_matrix': adiabatic_matrix.tolist(),
                       'diabatic_matrix': diabatic_matrix.tolist()}

        if diabat_section.find('showmatrix Total_Decomposed_H_diabatic'):

            tot_decomp_matrix = read_diabatization_matrix('showmatrix Total_Decomposed_H_diabatic') * AU_TO_EV
            decomp_one_matrix = read_diabatization_matrix('showmatrix Decomposed_One_diabatic') * AU_TO_EV
            decomp_j_matrix = read_diabatization_matrix('showmatrix Decomposed_J_diabatic') * AU_TO_EV
            decomp_k_matrix = read_diabatization_matrix('showmatrix Decomposed_K_diabatic') * AU_TO_EV

            diabat_data.update({'tot_decomp_matrix': tot_decomp_matrix,
                                               'decomp_one_matrix': decomp_one_matrix.tolist(),
                                               'decomp_j_matrix': decomp_j_matrix.tolist(),
                                               'decomp_k_matrix': decomp_k_matrix.tolist()})

        mulliken_diabatic = []

        enum = output.find('Mulliken & Loewdin analysis of')
        for m in re.finditer('Mulliken analysis of TDA State', output[enum:]):
            section_mulliken = output[m.end() + enum: m.end() + 10000 + enum]  # 10000: assumed to max of section
            section_mulliken = section_mulliken[:section_mulliken.find('Natural Orbitals stored in FCHK')]
            section_attachment = section_mulliken.split('\n')[10 + n_atoms: 10 + n_atoms * 2]

            mulliken_diabatic.append({'attach': [float(l.split()[1]) for l in section_attachment],
                                      'detach': [float(l.split()[2]) for l in section_attachment],
                                      'total': [float(l.split()[3]) for l in section_attachment]})

        diabatic_states = []
        for i in range(len(rot_matrix)):
            diabat_states_data = {'excitation_energy': diabatic_matrix[i][i],
                                  'excitation_energy_units': 'eV',
                                  'transition_moment': [],
                                  'dipole_moment_units': 'ua'}
            if len(mulliken_diabatic) > 0:
                diabat_states_data['mulliken'] = mulliken_diabatic[i]

            diabatic_states.append(diabat_states_data)
        diabat_data['diabatic_states'] = diabatic_states

        data_dict['diabatization'] = diabat_data

    return data_dict




from pyqchem.structure import Structure
import numpy as np
import re


def basic_irc(output, print_data=False):

    branch_mark = True
    data_dict = {}
    # Molecule
    n = output.find('$molecule')
    n2 = output[n:].find('$end')

    molecule_region = output[n:n+n2-1].replace('\t', ' ').split('\n')[1:]
    charge, multiplicity = [int(num) for num in molecule_region[0].split()]
    coordinates = np.array([np.array(line.split()[1:4], dtype=float) for line in molecule_region[1:]])
    symbols = [line.split()[0].capitalize() for line in molecule_region[1:]]
    n_atoms = len(coordinates)

    forward_steps = []
    backward_steps = []

    list_iterations = [l.end() for l in re.finditer('Reaction path following', output)]
    for ini, fin in zip(list_iterations, list_iterations[1:] + [len(output)]):
        step_section = output[ini:fin]
        enum = step_section.find('Standard Nuclear Orientation')
        atoms_list = step_section[enum:].split('\n')[3:n_atoms+3]
        coordinates_step = np.array([atom.split()[2:] for atom in atoms_list], dtype=float).tolist()

        step_energy = None
        for l in re.finditer('Total energy in the final basis set', step_section):
            step_energy = float(step_section[l.end(): l.end()+50].split()[1])

        step_molecule = Structure(coordinates=coordinates_step,
                                  symbols=symbols,
                                  charge=charge,
                                  multiplicity=multiplicity)

        if (step_section.find('IRC -- maximum number of cycles reached') > 0
                or step_section.find('IRC -- convergence criterion reached') > 0):
            if branch_mark:
                #forward_steps = forward_steps[::-1]
                branch_mark = False
            else:
                break

        if branch_mark:
            forward_steps.append({'molecule': step_molecule,
                                  'energy': step_energy,
                                  })
        else:
            backward_steps.append({'molecule': step_molecule,
                                   'energy': step_energy,
                                   })

    data_dict['irc_forward'] = forward_steps
    data_dict['irc_backward'] = forward_steps

    return data_dict


from pyqchem.qchem_core import parse_output
import numpy as np
import matplotlib.pyplot as plt

from pyqchem.parsers.parser_diabatic import analyze_diabatic
from pyqchem.parsers.basic import basic_parser_qchem
# common qchem input parameters


def get_order_states(states, epsilon=1):

    print([state['total energy'] for state in states])
    print([state['transition moment'] for state in states])
    print([state['transition moment'][1] for state in states])

    order = np.argsort([state['total energy'] for state in states]).tolist()

    for i, tm in enumerate([state['transition moment'] for state in states]):
        if np.linalg.norm(tm) > epsilon:
            index = order.pop(i)
            order = [index] + order

    order = np.argsort([np.abs(state['transition moment'][1]) for state in states]).tolist()[::-1]
    order[1], order[3] = order[3], order[1]
#    order[1], order[0] = order[0], order[1]

    print('--------------order------------\n', order)
    return order


def correct_order(list, order):

    alist = np.array(list)
    orddered_list = alist[order]

    return orddered_list


@parse_output
def get_output(filename):
    with open(filename, 'r') as f:
        output = f.read()
    return output, ''

dif1 = []
dif2 = []
dif3 = []

ener = []
ener1 = []
ener2 = []
ener3 = []

kill = []
kill2 = []
kill3 = []
kill4 = []
kill5 = []

points = []
points_total = [0, 0.25, 0.5, 0.75, 1, 1.25, 1.5, 1.75, 2, 2.25, 2.5, 2.75, 3, 3.25, 3.5, 3.75, 4]
for num in points_total:
    file = 'test_outputs_3/qchemout_0_{}.out'.format(num)

    print('file: {}'.format(file))
    data1 = get_output(file, parser=basic_parser_qchem, force_recalculation=True)

    try:
        states = np.array(get_output(file).split('\n')[180].split(), dtype=int)-1
    except ValueError:
        states = np.array(get_output(file).split('\n')[171].split(), dtype=int)-1

    # get list of interesting states
    states_info = []
    for state in states:
        states_info.append(data1['excited states cis'][state])
    print(states_info)

    order = get_order_states(states_info)
    print(states)

    e = np.array([data1['excited states cis'][state]['excitation energy'] for state in states])[order]
    kill3.append(e)

    t = np.array([np.linalg.norm(data1['excited states cis'][state]['transition moment']) for state in states])[order]
    kill4.append(t)


    # get energy contributions
    try:
        data = get_output(file, parser=analyze_diabatic, force_recalculation=True)
    except Exception:
        print('Failed!')
        continue

    coefficients = np.zeros([4, 4])
    adiabatic_energies = np.zeros(4)
    diabatic_energies = np.zeros([4, 4])
    for i in range(4):
        for j in range(4):
            coefficients[i, j] = data['coefficients']['S{}_C{}'.format(i + 1, j + 1)]
            diabatic_energies[i, j ] = data['diabatic_energies']['E_{}_{}'.format(i+1, j+1)]

        adiabatic_energies[i] = data['adiabatic_energies']['E_{}'.format(i + 1)]

    print('coefficients')
    print(coefficients)
    print('adiabatic energies')
    print(adiabatic_energies)
    print('diabatic')
    print(diabatic_energies)

    print('recalculated adiabatic energies')
    calculated_adiabatic = np.diag(np.dot(coefficients, np.dot(diabatic_energies, coefficients.T)))
    print(calculated_adiabatic)

    calculated_adiabatic = correct_order(calculated_adiabatic, order)
    adiabatic_energies = correct_order(adiabatic_energies, order)

    print('diference adiabatic (original - calculated)')
    print(calculated_adiabatic - adiabatic_energies)

    dif1.append(calculated_adiabatic - adiabatic_energies)
    ener1.append(calculated_adiabatic)
    ener.append(adiabatic_energies)

    print('-------------------------------------')

    diabatic_energies_2 = np.zeros([4, 4])

    diabatic_energies_2[0, 0] = np.average(data['diabatic_energies']['E_LE'])
    diabatic_energies_2[2, 2] = np.average(data['diabatic_energies']['E_CT'])

    diabatic_energies_2[0, 1] = data['diabatic_energies']['V_DC']
    diabatic_energies_2[2, 3] = data['diabatic_energies']['V_CT']

    diabatic_energies_2[1, 2] = data['diabatic_energies']['V_e'][0]
    diabatic_energies_2[1, 3] = data['diabatic_energies']['V_h'][0]

    # prima
    diabatic_energies_2[1, 1] = np.average(data['diabatic_energies']['E_LE'])
    diabatic_energies_2[3, 3] = np.average(data['diabatic_energies']['E_CT'])

    diabatic_energies_2[0, 3] = data['diabatic_energies']['V_e'][1]
    diabatic_energies_2[0, 2] = data['diabatic_energies']['V_h'][1]

    # fill lower diagonal matrix
    for i in range(4):
        for j in range(i):
            diabatic_energies_2[i, j] = diabatic_energies_2[j, i]

    # Diabatic matrix
    #   E_LE   V_DC   V_h'   V_e'
    #   V_DC   E_LE'  V_e    V_h
    #   V_h'   V_e    E_CT   V_e
    #   V_e'   V_h    V_e    E_CT'

    print('diabatic_energies_2')
    print(diabatic_energies_2)

    coefficients_2 = [data['coefficients']['S_01'],
                      data['coefficients']['S_10'],
                      data['coefficients']['S_CA'],
                      data['coefficients']['S_AC']]

    coefficients_2 = np.array(coefficients_2).T

    print('coefficients_2')
    print(coefficients_2)

    calculated_adiabatic_2 = np.diag(np.dot(coefficients_2, np.dot(diabatic_energies_2, coefficients_2.T)))
    print(calculated_adiabatic_2)

    calculated_adiabatic_2 = correct_order(calculated_adiabatic_2, order)

    print('diference adiabatic_2 (original - calculated)')
    print(calculated_adiabatic_2 - adiabatic_energies)

    dif2.append(calculated_adiabatic_2 - adiabatic_energies)
    ener2.append(calculated_adiabatic_2)

    print('-------------------------------------')

    calculated_adiabatic_3 = []
    w_e_list = []
    w_h_list = []
    lambda_list = []

    e_le = np.average(data['diabatic_energies']['E_LE'])
    e_ct = np.average(data['diabatic_energies']['E_CT'])

    for i in range(4):

        lmb = data['lambda'][i]

        w_dc = data['diabatic_energies']['V_DC'] * np.sign(data['coefficients']['S_10'][i] *
                                                           data['coefficients']['S_01'][i])

        w_ct = data['diabatic_energies']['V_CT'] * np.sign(data['coefficients']['S_CA'][i] *
                                                           data['coefficients']['S_AC'][i])

        w_h1 = data['diabatic_energies']['V_h'][0] * np.sign(data['coefficients']['S_10'][i] *
                                                             data['coefficients']['S_AC'][i])

        w_h2 = data['diabatic_energies']['V_h'][1] * np.sign(data['coefficients']['S_01'][i] *
                                                             data['coefficients']['S_CA'][i])

        w_e1 = data['diabatic_energies']['V_e'][0] * np.sign(data['coefficients']['S_10'][i] *
                                                             data['coefficients']['S_CA'][i])

        w_e2 = data['diabatic_energies']['V_e'][1] * np.sign(data['coefficients']['S_01'][i] *
                                                             data['coefficients']['S_AC'][i])

        w_e = np.average([w_e1, w_e2])
        w_h = np.average([w_h1, w_h2])

        o_e = 2 * lmb * np.sqrt(1 - lmb ** 2) * w_e
        o_h = 2 * lmb * np.sqrt(1 - lmb ** 2) * w_h

        #w_dc = data['diabatic_contributions']['W_DC'][i]
        #w_ct = data['diabatic_contributions']['W_CT'][i]

        #w_e = data['diabatic_contributions']['W_e'][i]
        #w_h = data['diabatic_contributions']['W_h'][i]

        #o_e = data['diabatic_contributions']['Omega_e'][i]
        #o_h = data['diabatic_contributions']['Omega_h'][i]

        ###### manual fix ########
        manual_fix = False
        if manual_fix:
            if num < 0.9:
                kk2 = -1
            else:
                kk2 = 1

            if num > 2.2:
                kk3 = -1
            else:
                kk3 = 1

            if i == order[0]:
                lmb = lmb * kk2
                w_h = w_h * kk2
                w_e = w_e * kk2

            #if j == order[1]:
            #    w_h = w_h * kk2
            #    w_e = w_e * kk2

            #if j == order[2]:
            #    w_h = w_h * kk3
            #    w_e = w_e * kk3

            if i == order[1]:
                lmb = lmb * kk3
                w_h = w_h * kk3
                w_e = w_e * kk3
        ###########

        w_e_list.append(w_e)
        w_h_list.append(w_h)
        lambda_list.append(lmb)

        calculated_adiabatic_3.append(e_le + w_dc + 2 * lmb * np.sqrt(1 - lmb ** 2) * (w_e + w_h) + lmb ** 2 * (e_ct - e_le + w_ct - w_dc))

    calculated_adiabatic_3 = correct_order(calculated_adiabatic_3, order)
    w_e_list = correct_order(w_e_list, order)
    w_h_list = correct_order(w_h_list, order)
    lambda_list = correct_order(lambda_list, order)

    kill.append(w_e_list)
    kill2.append(w_h_list)
    kill5.append(lambda_list)

    ener3.append(calculated_adiabatic_3)
    dif3.append(calculated_adiabatic_3 - adiabatic_energies)

    points.append(num)

plt.figure()
plt.title('excitation energy')
for i, k in enumerate(np.array(kill3).T[:]):
    plt.plot(points_total, k, '-o', label=i)
plt.legend()

plt.figure()
plt.title('transition moment module')
for i, k in enumerate(np.array(kill4).T[:]):
    plt.plot(points_total, k, '-o', label=i)
plt.legend()
plt.show()


#exit()
plt.figure()
plt.title('lambda')

for i, k in enumerate(np.array(kill5).T[2:]):
    plt.plot(points, k, '-o', label=i)
plt.legend()

for i, k in enumerate(np.array(kill5).T[:2]):
    plt.plot(points, k, '--', label=i+2)
plt.legend()

plt.figure()
plt.title('W_e')

for i, k in enumerate(np.array(kill).T[:2]):
    plt.plot(points, k, '--', label=i)
plt.legend()

for i, k in enumerate(np.array(kill).T[2:]):
    plt.plot(points, k, '-o', label=i+2)
plt.legend()


plt.figure()
plt.title('W_h')

for i, k in enumerate(np.array(kill2).T[:2]):
    plt.plot(points, k, '--', label=i)
plt.legend()

for i, k in enumerate(np.array(kill2).T[2:]):
    plt.plot(points, k, '-o', label=i+2)
plt.legend()


plt.show()

#exit()

dif1 = np.array(dif1).T
dif2 = np.array(dif2).T
dif3 = np.array(dif3).T

ener1 = np.array(ener1).T
ener2 = np.array(ener2).T
ener3 = np.array(ener3).T

ener = np.array(ener).T


plt.title('diference adiabatic')
for i, p in enumerate(dif1):
    plt.plot(points, p, '-o',label='{}: {}'.format('Direct', i+1))

for i, p in enumerate(dif2):
    plt.plot(points, p, '-+', label='{}: {}'.format('Separated', i+1))

for i, p in enumerate(dif3):
    plt.plot(points, p, '-x', label='{}: {}'.format('Parts', i+1))

plt.legend()
plt.show()

#exit()

plt.title('Adiabatic energies')
for i, p in enumerate(ener1):
    plt.plot(points, p, '-o', label='{}: {}'.format('Direct', i+1))

for i, p in enumerate(ener2):
    plt.plot(points, p, '-+', label='{}: {}'.format('Separated', i+1))

for i, p in enumerate(ener3):
    plt.plot(points, p, '-x', label='{}: {}'.format('Parts', i+1))

for i, p in enumerate(ener):
    plt.plot(points, p, '--', label='{}: {}'.format('Reference', i+1))

plt.legend()
plt.show()


